import React, { Component } from 'react';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

/* global Paho */

var SnipsMqttServer = function () {
    function SnipsMqttServer(props) {
        classCallCheck(this, SnipsMqttServer);

        this.props = props;
        this.failCount = 0;
        this.mqttClient = null;
        this.sessionId = null;
        this.clientId = props.clientId ? props.clientId : 'client' + parseInt(Math.random() * 100000000, 10);
        this.state = {};
        this.connectTimeout = null;
        this.started = false;
        this.onMessageArrived = this.onMessageArrived.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.onConnectionLost = this.onConnectionLost.bind(this);
        this.mqttConnect = this.mqttConnect.bind(this);
    }

    createClass(SnipsMqttServer, [{
        key: 'setState',
        value: function setState(newState) {
            return Object.assign(this.state, newState);
        }
    }, {
        key: 'start',
        value: function start() {
            this.started = true;
            this.mqttConnect();
        }
    }, {
        key: 'stop',
        value: function stop() {
            this.started = false;
            if (this.mqttClient) this.mqttClient.disconnect();
        }
    }, {
        key: 'mqttConnect',


        /**
          * Connect to mqtt server
         */
        value: function mqttConnect() {
            var port = this.props.mqttPort && this.props.mqttPort > 0 ? parseInt(this.props.mqttPort, 10) : 9001;
            var server = this.props.mqttServer && this.props.mqttServer.length > 0 ? this.props.mqttServer : window.location.hostname;
            this.mqttClient = new Paho.MQTT.Client(server, port, this.clientId);
            this.mqttClient.onConnectionLost = this.onConnectionLost;
            this.mqttClient.onMessageArrived = this.onMessageArrived;
            this.mqttClient.connect({ onSuccess: this.onConnect });
        }
    }, {
        key: 'onConnect',


        /**
         * Subscribe to to mqtt channels then start recorder
         */
        value: function onConnect() {
            var that = this;
            //console.log([' SERVER CONNECTED',this.eventFunctions]);
            this.setState({ 'connected': true });
            this.failCount = 0;
            var subscribe = this.props.subscribe && this.props.subscribe.length > 0 ? this.props.subscribe : '#';
            that.mqttClient.subscribe(subscribe, {});
            this.afterConnect(this);
        }
    }, {
        key: 'afterConnect',
        value: function afterConnect() {
            //console.log([' after CONNECTED']);
        }
    }, {
        key: 'onConnectionLost',


        /**
         * When the client loses its connection, reconnect after 5s
         */
        value: function onConnectionLost(responseObject) {
            var that = this;
            this.setState({ 'connected': false });
            if (responseObject.errorCode !== 0) {
                console.log(["  SERVER onConnectionLost:" + responseObject.errorMessage]);
                if (this.props.onConnectionLost && typeof this.props.onConnectionLost === "function") {
                    this.props.onConnectionLost(this);
                } else {
                    var timeout = 1000;
                    if (this.failCount > 5) {
                        timeout = 10000;
                    } else if (this.failCount > 10) {
                        timeout = 30000;
                    } else if (this.failCount > 15) {
                        timeout = 360000;
                    }
                    this.failCount++;
                    clearTimeout(this.connectTimeout);
                    this.connectTimeout = setTimeout(function () {
                        that.mqttClient.connect({ onSuccess: that.onConnect });
                    }, timeout);
                }
            }
        }
    }, {
        key: 'onMessageArrived',


        // respond to hermes/hotword/toggleOn or toggleOff where the siteId matches props.siteId
        value: function onMessageArrived(message) {
            // console.log(['IMPLEMENT ONMESSAGE',message]);
        }
    }]);
    return SnipsMqttServer;
}();

var eventFunctions = {

    /* Hotword */
    'hermes/hotword/toggleOn': function hermesHotwordToggleOn(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            var hotwordListening = that.state.hotwordListening;
            hotwordListening[payload.siteId] = true;
            that.setState({ hotwordListening: hotwordListening });
            that.updateSession(payload, function (session) {
                if (session) {
                    session.hotword = true;
                    session.hotwordDetected = false;
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/hotword/toggleOff': function hermesHotwordToggleOff(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            var hotwordListening = that.state.hotwordListening;
            hotwordListening[payload.siteId] = false;
            that.setState({ hotwordListening: hotwordListening });
            that.updateSession(payload, function (session) {
                if (session) {
                    session.hotword = false;
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/hotword/#/detected': function hermesHotwordDetected(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    session.hotwordDetected = true;
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },

    /* NLU */
    'hermes/nlu/query': function hermesNluQuery(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/nlu/partialQuery': function hermesNluPartialQuery(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/nlu/intentParsed': function hermesNluIntentParsed(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/nlu/intentNotRecognized': function hermesNluIntentNotRecognized(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/nlu/slotParsed': function hermesNluSlotParsed(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/error/nlu': function hermesErrorNlu(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },

    /* TTS */
    'hermes/tts/say': function hermesTtsSay(payload) {
        var that = this;
        //console.log(['LOG TTS']);
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    if (!session.tts) session.tts = [];
                    session.tts.push(payload);
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },

    'hermes/tts/sayFinished': function hermesTtsSayFinished(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },

    'hermes/asr/toggleOn': function hermesAsrToggleOn(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/asr/toggleOff': function hermesAsrToggleOff(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/asr/startListening': function hermesAsrStartListening(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.resetAudioBuffer(payload.siteId);
            var audioListening = that.state.audioListening;
            audioListening[payload.siteId] = true;
            that.setState({ audioListening: audioListening });
            that.updateSession(payload, function (session) {
                if (session && payload) that.updateSessionStatus(payload.siteId, session);
                return session;
            });
            resolve();
            // console.log(['START ASR']);
        });
    },
    'hermes/asr/stopListening': function hermesAsrStopListening(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            var audioListening = that.state.audioListening;
            audioListening[payload.siteId] = false;
            that.setState({ audioListening: audioListening });
            // log audio
            that.updateSession(payload, function (session) {
                if (session) {
                    that.logAudioBuffer(payload);
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/asr/textCaptured': function hermesAsrTextCaptured(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    if (!session.asr) session.asr = [];
                    session.asr.push(payload);
                }
                return session;
            });
            resolve();
        });
    },

    'hermes/asr/partialTextCaptured': function hermesAsrPartialTextCaptured(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },

    'hermes/dialogueManager/startSession': function hermesDialogueManagerStartSession(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/dialogueManager/continueSession': function hermesDialogueManagerContinueSession(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/dialogueManager/endSession': function hermesDialogueManagerEndSession(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/dialogueManager/sessionStarted': function hermesDialogueManagerSessionStarted(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    session.started = true;
                    session.queued = true;
                    session.starttimestamp = new Date().getTime();
                    session.sessionId = payload.sessionId;
                    session.siteId = payload.siteId;
                    that.updateSessionStatus(payload.siteId, session);
                    that.lastSessionId[payload.siteId] = payload.sessionId;
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/dialogueManager/sessionEnded': function hermesDialogueManagerSessionEnded(payload) {
        var that = this;
        // console.log(['SESSION ENDED',payload]);
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    if (session.termination && session.termination.reason && session.termination.reason === "nominal") {
                        session.success = true;
                    }
                    session.ended = true;
                    session.endtimestamp = new Date().getTime();
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/dialogueManager/sessionQueued': function hermesDialogueManagerSessionQueued(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    session.queued = true;
                    session.queuedtimestamp = new Date().getTime();
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },
    'hermes/intent/#': function hermesIntent(payload) {
        var that = this;
        return new Promise(function (resolve, reject) {
            that.updateSession(payload, function (session) {
                if (session) {
                    if (!session.intents) session.intents = [];
                    session.intents.push(payload);
                    that.updateSessionStatus(payload.siteId, session);
                }
                return session;
            });
            resolve();
        });
    },

    /* Feedback *
     */
    'hermes/feedback/sound/toggleOn': function hermesFeedbackSoundToggleOn(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/feedback/sound/toggleOff': function hermesFeedbackSoundToggleOff(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/audioServer/#/playBytes': function hermesAudioServerPlayBytes(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/audioServer/#/playFinished': function hermesAudioServerPlayFinished(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    },
    'hermes/audioServer/#/audioFrame': function hermesAudioServerAudioFrame(payload) {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    }

};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x.default : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var crunker = createCommonjsModule(function (module, exports) {
!function(e,t){module.exports=t();}(window,function(){return function(e){var t={};function n(r){if(t[r])return t[r].exports;var o=t[r]={i:r,l:!1,exports:{}};return e[r].call(o.exports,o,o.exports,n),o.l=!0,o.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r});},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0});},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var o in e)n.d(r,o,function(t){return e[t]}.bind(null,o));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=0)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r);}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}();var o=function(){function e(){var t=(arguments.length>0&&void 0!==arguments[0]?arguments[0]:{}).sampleRate,n=void 0===t?44100:t;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this._sampleRate=n,this._context=this._createContext();}return r(e,[{key:"_createContext",value:function(){return window.AudioContext=window.AudioContext||window.webkitAudioContext||window.mozAudioContext,new AudioContext}},{key:"fetchAudio",value:async function(){for(var e=this,t=arguments.length,n=Array(t),r=0;r<t;r++)n[r]=arguments[r];var o=n.map(async function(t){var n=await fetch(t).then(function(e){return e.arrayBuffer()});return await e._context.decodeAudioData(n)});return await Promise.all(o)}},{key:"mergeAudio",value:function(e){var t=this._context.createBuffer(1,this._sampleRate*this._maxDuration(e),this._sampleRate);return e.map(function(e){for(var n=e.getChannelData(0).length-1;n>=0;n--)t.getChannelData(0)[n]+=e.getChannelData(0)[n];}),t}},{key:"concatAudio",value:function(e){var t=this._context.createBuffer(1,this._totalLength(e),this._sampleRate),n=0;return e.map(function(e){t.getChannelData(0).set(e.getChannelData(0),n),n+=e.length;}),t}},{key:"play",value:function(e){var t=this._context.createBufferSource();return t.buffer=e,t.connect(this._context.destination),t.start(),t}},{key:"export",value:function(e,t){var n=t||"audio/mp3",r=this._interleave(e),o=this._writeHeaders(r),i=new Blob([o],{type:n});return {blob:i,url:this._renderURL(i),element:this._renderAudioElement(i,n)}}},{key:"download",value:function(e,t){var n=t||"crunker",r=document.createElement("a");return r.style="display: none",r.href=this._renderURL(e),r.download=n+"."+e.type.split("/")[1],r.click(),r}},{key:"notSupported",value:function(e){return !this._isSupported()&&e()}},{key:"close",value:function(){return this._context.close(),this}},{key:"_maxDuration",value:function(e){return Math.max.apply(Math,e.map(function(e){return e.duration}))}},{key:"_totalLength",value:function(e){return e.map(function(e){return e.length}).reduce(function(e,t){return e+t},0)}},{key:"_isSupported",value:function(){return "AudioContext"in window}},{key:"_writeHeaders",value:function(e){var t=new ArrayBuffer(44+2*e.length),n=new DataView(t);return this._writeString(n,0,"RIFF"),n.setUint32(4,32+2*e.length,!0),this._writeString(n,8,"WAVE"),this._writeString(n,12,"fmt "),n.setUint32(16,16,!0),n.setUint16(20,1,!0),n.setUint16(22,2,!0),n.setUint32(24,this._sampleRate,!0),n.setUint32(28,4*this._sampleRate,!0),n.setUint16(32,4,!0),n.setUint16(34,16,!0),this._writeString(n,36,"data"),n.setUint32(40,2*e.length,!0),this._floatTo16BitPCM(n,e,44)}},{key:"_floatTo16BitPCM",value:function(e,t,n){for(var r=0;r<t.length;r++,n+=2){var o=Math.max(-1,Math.min(1,t[r]));e.setInt16(n,o<0?32768*o:32767*o,!0);}return e}},{key:"_writeString",value:function(e,t,n){for(var r=0;r<n.length;r++)e.setUint8(t+r,n.charCodeAt(r));}},{key:"_interleave",value:function(e){for(var t=e.getChannelData(0),n=2*t.length,r=new Float32Array(n),o=0,i=0;o<n;)r[o++]=t[i],r[o++]=t[i],i++;return r}},{key:"_renderAudioElement",value:function(e,t){var n=document.createElement("audio");return n.controls="controls",n.type=t,n.src=this._renderURL(e),n}},{key:"_renderURL",value:function(e){return (window.URL||window.webkitURL).createObjectURL(e)}}]),e}();t.default=o;}])});
});

var Crunker = unwrapExports(crunker);
var crunker_1 = crunker.Crunker;

/* global window */

var SnipsLogger$1 = function (_SnipsMqttServer) {
    inherits(SnipsLogger, _SnipsMqttServer);

    function SnipsLogger(props) {
        classCallCheck(this, SnipsLogger);

        var _this = possibleConstructorReturn(this, (SnipsLogger.__proto__ || Object.getPrototypeOf(SnipsLogger)).call(this, props));

        _this.eventFunctions = eventFunctions;
        _this.eventCallbackFunctions = _this.addCallbacks(_this.props.eventCallbackFunctions);
        _this.siteId = props.siteId ? props.siteId : 'site' + parseInt(Math.random() * 100000000, 10);
        _this.state = { sites: {}, messages: [], session: {}, audioListening: {}, hotwordListening: {}, showLogMessages: {}, sessionStatus: {}, sessionStatusText: {} };
        _this.audioBuffers = {};
        _this.lastSessionId = {};

        _this.getSession = _this.getSession.bind(_this);
        _this.saveSession = _this.saveSession.bind(_this);
        _this.updateSession = _this.updateSession.bind(_this);
        _this.logAudioBuffer = _this.logAudioBuffer.bind(_this);
        _this.updateSessionStatus = _this.updateSessionStatus.bind(_this);
        _this.addCallbacks = _this.addCallbacks.bind(_this);
        _this.findEventCallbackFunctions = _this.findEventCallbackFunctions.bind(_this);
        _this.onMessageArrived = _this.onMessageArrived.bind(_this);
        _this.isConnected = _this.isConnected.bind(_this);
        _this.reset = _this.reset.bind(_this);
        _this.mqttConnect.bind(_this)();
        return _this;
    }

    createClass(SnipsLogger, [{
        key: 'reset',
        value: function reset() {
            this.setState({ sites: {}, messages: [], session: {}, audioListening: {}, hotwordListening: {}, showLogMessages: {}, sessionStatus: {}, sessionStatusText: {} });
            this.audioBuffers = {};
            this.lastSessionId = {};
        }
    }, {
        key: 'addCallbacks',
        value: function addCallbacks(eventCallbackFunctions) {
            var oneOff = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            var that = this;
            this.eventCallbackFunctions = Array.isArray(this.eventCallbackFunctions) ? this.eventCallbackFunctions : [];
            if (eventCallbackFunctions) {
                Object.keys(eventCallbackFunctions).map(function (key, loopKey) {
                    var value = eventCallbackFunctions[key];
                    if (typeof value === "function") {
                        that.eventCallbackFunctions.push({ subscription: key, callBack: value, oneOff: oneOff, id: parseInt(Math.random() * 100000000, 10) });
                    }
                });
            }
            return this.eventCallbackFunctions;
        }
    }, {
        key: 'findEventCallbackFunctions',
        value: function findEventCallbackFunctions(subscriptionKey) {
            var ret = [];
            this.eventCallbackFunctions.map(function (value, vkey) {
                if (value.subscription === subscriptionKey) {
                    ret.push(value);
                    return;
                }
            });
            return ret;
        }
    }, {
        key: 'generateUuid',
        value: function generateUuid() {
            //// return uuid of form xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
            var uuid = '',
                ii;
            for (ii = 0; ii < 32; ii += 1) {
                switch (ii) {
                    case 8:
                    case 20:
                        uuid += '-';
                        uuid += (Math.random() * 16 | 0).toString(16);
                        break;
                    case 12:
                        uuid += '-';
                        uuid += '4';
                        break;
                    case 16:
                        uuid += '-';
                        uuid += (Math.random() * 4 | 8).toString(16);
                        break;
                    default:
                        uuid += (Math.random() * 16 | 0).toString(16);
                }
            }
            return uuid;
        }
    }, {
        key: 'onMessageArrived',
        value: function onMessageArrived(message) {
            var that = this;
            var parts = message.destinationName ? message.destinationName.split("/") : [];
            if (parts.length > 0 && parts[0] === "hermes") {
                if (parts.length > 1 && parts[1] === "audioServer") {
                    var audio = message.payloadBytes;
                    if (parts.length > 3) {
                        var siteId = parts[2];
                        var action = parts[3];
                        var id = parts.length > 4 ? parts[4] : ''; //this.generateUuid() ;
                        if (action === "playBytes") ; else if (action === "playFinished") ; else if (action === "audioFrame") {
                            this.appendAudioBuffer(siteId, audio);
                        }
                        var functionKey = 'hermes/audioServer/#/' + action;
                        // hooks and callbacks
                        // if (action !== "audioFrame") console.log(['LOGGER AUDIO CALLBACK',functionKey,that.eventFunctions[functionKey]]);

                        if (this.eventFunctions.hasOwnProperty(functionKey)) {
                            var p = that.eventFunctions[functionKey].bind(that)(audio);
                            p.then(function (session) {
                                //if (action !== "audioFrame")  console.log(['LOGGER AUDIO CALLBACK COMPLETE',functionKey,session]);
                                var callbacks = that.findEventCallbackFunctions(functionKey);
                                //if (action !== "audioFrame")  console.log(['LOGGER AUDIO CALLBACKS',callbacks]);
                                if (callbacks) {
                                    callbacks.map(function (value, ckey) {
                                        //if (action !== "audioFrame")  console.log(['LOGGER AUDIO CALLBACK ONE',value,ckey]);
                                        var session = that.getSession(siteId, null);
                                        value.callBack.bind(that)(message.destinationName, siteId, id, session, audio);
                                    });
                                }
                            }).catch(function (e) {
                                console.log(e);
                            });
                        }
                        var sessionId = this.lastSessionId[siteId];

                        var messages = this.state.messages;
                        if (action !== "audioFrame" && (!this.props.siteId || this.props.siteId && this.props.siteId === siteId)) {
                            messages.push({ sessionId: sessionId, payload: React.createElement(
                                    'div',
                                    { style: { backgroundColor: 'lightgrey' } },
                                    React.createElement('hr', null)
                                ), text: message.destinationName });
                            this.setState({ messages: messages });
                        }
                    }
                } else {
                    var payload = {};
                    try {
                        payload = JSON.parse(message.payloadString);
                    } catch (e) {}
                    // special case for hotword parameter in url
                    var _functionKey = message.destinationName;
                    if (parts.length > 3 && parts[0] === "hermes" && parts[1] === "hotword" && parts[3] === "detected") {
                        _functionKey = 'hermes/hotword/#/detected';
                        // special case for intent parameter in hermes/intent
                    } else if (parts.length > 1 && parts[0] === "hermes" && parts[1] === "intent") {
                        _functionKey = 'hermes/intent/#';
                    }

                    if (this.eventFunctions.hasOwnProperty(_functionKey)) {
                        var _p = that.eventFunctions[_functionKey].bind(that)(payload);
                        _p.then(function (session) {
                            var callbacks = that.findEventCallbackFunctions(_functionKey);
                            if (callbacks) {
                                callbacks.map(function (value, ckey) {
                                    value.callBack.bind(that)(payload, message);
                                    if (value.oneOff) {
                                        // remove this callback
                                        var breakLoop = false;
                                        that.eventCallbackFunctions.map(function (tvalue, vkey) {
                                            if (value.id === tvalue.id && !breakLoop) {
                                                that.eventCallbackFunctions.splice(vkey, 1);
                                                breakLoop = true;
                                            }
                                            return;
                                        });
                                    }
                                });
                            }
                        }).catch(function (e) {
                            console.log(e);
                        });
                    }

                    var _messages = this.state.messages;
                    var thisState = {};
                    if (payload.siteId) {
                        if (payload.sessionId) {
                            thisState = this.getSession(payload.siteId, payload.sessionId);
                        }
                    }
                    _messages.push({ sessionId: thisState.sessionId, payload: React.createElement(
                            'div',
                            { style: { backgroundColor: 'lightgrey' } },
                            React.createElement('hr', null),
                            React.createElement(
                                'div',
                                { style: { backgroundColor: 'lightblue' } },
                                React.createElement(
                                    'pre',
                                    null,
                                    JSON.stringify(payload, undefined, 4)
                                )
                            ),
                            React.createElement('hr', null),
                            React.createElement(
                                'div',
                                { style: { backgroundColor: 'lightgreen' } },
                                React.createElement(
                                    'pre',
                                    null,
                                    JSON.stringify(thisState, undefined, 4)
                                )
                            ),
                            React.createElement('hr', null)
                        ), text: message.destinationName });
                    this.setState({ messages: _messages });

                    // console.log(['LOGGER MESSAGE',message.destinationName,message,JSON.parse(JSON.stringify(this.state.sites))]);
                }
            }
        }
    }, {
        key: 'cleanSlots',
        value: function cleanSlots(slots) {
            var final = {};
            if (slots) {
                slots.map(function (slot) {
                    final[slot.slotName] = { type: slot.value.kind, value: slot.value.value };
                    return;
                });
            }
            return final;
        }
    }, {
        key: 'getSession',


        /** 
         * Get a session for a given siteId and sessionId
         * 
         */
        //  that.lastSessionId[payload.siteId]
        value: function getSession(siteIdIn, sessionIdIn) {
            var siteId = null;
            var sessionId = null;
            var that = this;

            function findOrCreateSession(siteId, sessionId) {
                if (siteId && siteId.length > 0 && sessionId && sessionId.length > 0) {
                    if (that.state.sites && that.state.sites.hasOwnProperty(siteId) && that.state.sites[siteId].hasOwnProperty(sessionId) && that.state.sites[siteId][sessionId]) {
                        return that.state.sites[siteId][sessionId];
                    } else {
                        // fallback, create a new session
                        var sites = that.state.sites ? that.state.sites : {};
                        var sessions = that.state.sessions ? that.state.sessions : {};
                        if (!sites.hasOwnProperty(siteId)) sites[siteId] = {};
                        var newSession = { createtimestamp: new Date().getTime(), siteId: siteId, sessionId: sessionId };
                        if (!sites[siteId].hasOwnProperty(sessionId)) sites[siteId][sessionId] = newSession;
                        sessions[sessionId] = siteId;
                        that.setState({ sites: sites, sessions: sessions });
                        return sites[siteId][sessionId];
                    }
                } else {
                    console.log(['CANNOT FIND OR CREATE SESSION MISSING SITE OR SESSION ID', siteId, sessionId]);
                }
            }
            if (siteIdIn && siteIdIn.length > 0) {
                if (sessionIdIn && sessionIdIn.length > 0) {
                    // have site and session id
                    siteId = siteIdIn;
                    sessionId = sessionIdIn;
                } else {
                    // have siteId
                    siteId = siteIdIn;
                    sessionId = this.lastSessionId[siteId];
                }
            } else {
                if (sessionIdIn && sessionIdIn.length > 0) {
                    // have sessionId
                    sessionId = sessionIdIn;
                    siteId = this.state.sessions[sessionId];
                } else {
                    // have no ids
                    console.log(['FAILED TO CAPTURE MESSAGE WITH NO SITE OR SESSION ID']);
                    return null;
                }
            }
            return findOrCreateSession(siteId, sessionId);
        }
    }, {
        key: 'updateSession',


        /**
         *  Lookup session, use callback to make changes and restore session to state
         */
        value: function updateSession(payload, callback) {
            var siteId = payload && payload.siteId && payload.siteId.length > 0 ? payload.siteId : null;

            var sessionId = payload && payload.sessionId && payload.sessionId.length > 0 ? payload.sessionId : null;
            var session = this.getSession(siteId, sessionId);
            if (session) {
                var result = callback(session);
                this.saveSession(session.siteId, session.sessionId, result);
            }
        }
    }, {
        key: 'saveSession',
        value: function saveSession(siteIdIn, sessionIdIn, session) {
            var sessionId = sessionIdIn && sessionIdIn.length > 0 ? sessionIdIn : 'unknownSession';
            // ensure siteId
            var siteId = siteIdIn;
            if (!siteIdIn || siteIdIn.length === 0) {
                siteId = this.state.sessions[sessionId];
            }
            if (siteId && siteId.length > 0) {
                var sites = this.state.sites;
                sites[siteId][sessionId] = session;
                this.setState({ sites: sites });
                this.setLogData();
            }
        }
    }, {
        key: 'setLogData',
        value: function setLogData() {
            if (this.props.setLogData) this.props.setLogData(this.state.sites, this.state.messages, this.state.sessionStatus, this.state.sessionStatusText, this.state.hotwordListening, this.state.audioListening);
        }
    }, {
        key: 'updateSessionStatus',
        value: function updateSessionStatus(siteKey, session) {
            var that = this;
            var sessionStatus = 0;
            var sessionKey = session.sessionId;
            if (that.state.hotwordListening[siteKey]) sessionStatus = 1;
            if (that.state.audioListening[siteKey]) sessionStatus = 2;
            if (session.queued) sessionStatus = 3;
            if (session.started) sessionStatus = 4;
            if (session.intents && session.asr && session.intents.length < session.asr.length) sessionStatus = 5;
            if (session.intents && session.asr && session.intents.length === session.asr.length) sessionStatus = 6;
            if (session.ended) sessionStatus = 7;
            if (session.success) sessionStatus = 8;
            var statusTexts = ['starting', 'hotword', 'listening', 'queued', 'started', 'transcribed', 'interpreted', 'ended', 'success'];
            var statusText = statusTexts[sessionStatus];
            var allSessionsStatus = that.state.sessionStatus;
            var allSessionsStatusText = that.state.sessionStatusText;
            allSessionsStatus[sessionKey] = sessionStatus;
            allSessionsStatusText[sessionKey] = statusText;
            that.setState({ sessionStatus: allSessionsStatus, sessionStatusText: allSessionsStatusText });
        }
    }, {
        key: 'getAudioBuffer',


        /**
         * Get or create an audio buffer for the siteId
         */
        value: function getAudioBuffer(siteId) {
            if (siteId) {
                if (this.audioBuffers.hasOwnProperty(siteId)) {
                    return this.audioBuffers[siteId];
                } else {
                    this.audioBuffers[siteId] = [];
                    return this.audioBuffers[siteId];
                }
            }
        }
    }, {
        key: 'appendAudioBuffer',


        /**
         * Get or create an audio buffer for the siteId
         */
        value: function appendAudioBuffer(siteId, buffer) {

            if (this.props.logAudio === true && siteId) {
                if (this.state.audioListening[siteId]) {
                    var currentBuffer = this.getAudioBuffer(siteId);
                    currentBuffer.push(buffer);
                }
            }
        }
    }, {
        key: 'resetAudioBuffer',


        /**
         * Get or create an audio buffer for the siteId
         */
        value: function resetAudioBuffer(siteId) {
            this.audioBuffers[siteId] = [];
        }
    }, {
        key: 'logAudioBuffer',
        value: function logAudioBuffer(payload) {
            var that = this;
            var promises = [];
            var siteId = payload.siteId;

            if (this.props.logAudio === true) {
                try {
                    // save to sites/sessions
                    var audioContext = window.AudioContext || window.webkitAudioContext;
                    var context = new audioContext();
                    var audioBuffer = this.getAudioBuffer(siteId);
                    // memory overload protection
                    // if (audioBuffer.length> 350) return;
                    audioBuffer.map(function (bytes, key) {
                        var p = new Promise(function (resolve, reject) {
                            var buffer = new Uint8Array(bytes.length);
                            if (bytes.length > 0) {
                                buffer.set(new Uint8Array(bytes), 0);
                                try {
                                    context.decodeAudioData(buffer.buffer, function (audioBuffer) {
                                        resolve(audioBuffer);
                                    });
                                } catch (e) {
                                    // trash buffer
                                    reject();
                                }
                            }
                        });
                        promises.push(p);
                        return;
                    });
                } catch (e) {
                    console.log(['ERROR', e]);
                }

                Promise.all(promises).then(function (allBuffers) {
                    var merger = new Crunker();
                    try {
                        var output = merger.export(merger.concatAudio(allBuffers), "audio/wav");
                        that.updateSession(payload, function (session) {
                            if (!session.audio) session.audio = [];
                            that.blobToDataUrl(output.blob).then(function (dataUrl) {
                                session.audio.push(dataUrl);
                                // start again
                                that.resetAudioBuffer(siteId);
                                that.setLogData();
                            });
                            return session;
                        });
                    } catch (e) {
                        console.log(e.message);
                    }
                });
            }
        }
    }, {
        key: 'blobToDataUrl',
        value: function blobToDataUrl(blob) {
            return new Promise(function (fulfill, reject) {
                var reader = new FileReader();
                reader.onerror = reject;
                reader.onload = function (e) {
                    return fulfill(reader.result);
                };
                reader.readAsDataURL(blob);
            });
        }
    }, {
        key: 'sendMqtt',
        value: function sendMqtt(destination, payload) {

            //if (!destination.startsWith('hermes/audioServer')) console.log(['SESSION SEND MQTT LOGGER',destination,payload])
            if (this.state.connected) {
                var message = new Paho.MQTT.Message(JSON.stringify(payload));
                message.destinationName = destination;
                this.mqttClient.send(message);
            }
        }
    }, {
        key: 'sendAudioMqtt',
        value: function sendAudioMqtt(destination, payload) {
            if (this.state.connected) {
                var message = new Paho.MQTT.Message(payload);
                message.destinationName = destination;
                this.mqttClient.send(message);
            }
        }
    }, {
        key: 'say',


        /**
         * Send Mqtt message to sppeak text
         */
        value: function say(siteId, text) {
            var that = this;
            if (that.state && that.state.connected) {
                var currentSession = that.getSession(siteId, null);
                var payload = { siteId: siteId, text: text };
                if (currentSession) payload.sessionId = currentSession.sessionId;
                var message = new Paho.MQTT.Message(JSON.stringify(payload));
                message.destinationName = "hermes/tts/say";
                that.mqttClient.send(message);
            }
        }
    }, {
        key: 'isConnected',
        value: function isConnected() {
            return this.state.connected;
        }
    }]);
    return SnipsLogger;
}(SnipsMqttServer);

var SnipsReactComponent = function (_Component) {
    inherits(SnipsReactComponent, _Component);

    function SnipsReactComponent(props) {
        classCallCheck(this, SnipsReactComponent);

        var _this = possibleConstructorReturn(this, (SnipsReactComponent.__proto__ || Object.getPrototypeOf(SnipsReactComponent)).call(this, props));

        _this.props = props;
        _this.state = {};
        _this.logger = null;
        _this.queueOneOffCallbacks = _this.queueOneOffCallbacks.bind(_this);
        _this.connectToLogger = _this.connectToLogger.bind(_this);
        return _this;
    }

    createClass(SnipsReactComponent, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.connectToLogger(this.props.logger, {});
        }
    }, {
        key: 'connectToLogger',
        value: function connectToLogger(logger, eventFunctions) {
            if (logger) {
                logger.addCallbacks(eventFunctions);
                this.logger = logger;
                return this.logger;
            } else {
                this.logger = new SnipsLogger$1(Object.assign({ logAudio: false, setLogData: this.setLogData, eventCallbackFunctions: eventFunctions }, this.props));
                return this.logger;
            }
        }
    }, {
        key: 'queueOneOffCallbacks',
        value: function queueOneOffCallbacks(eventFunctions) {
            if (this.logger) {
                this.logger.addCallbacks(eventFunctions, true);
            }
        }
    }, {
        key: 'sendMqtt',
        value: function sendMqtt(destination, payload) {
            if (this.logger) {
                this.logger.sendMqtt(destination, payload);
            }
        }
    }, {
        key: 'setLogData',

        // force update
        value: function setLogData(sites, messages, sessionStatus, sessionStatusText, hotwordListening, audioListening) {
            this.setState(this.state);
        }
    }, {
        key: 'sendHotwordToggleOn',


        /**
         *  MQTT SEND 
         */

        /**
         * Send Mqtt message to toggle on hotword
         * Used to forcibly initialise the local hotword server.
         */
        value: function sendHotwordToggleOn(siteId) {
            this.sendMqtt("hermes/hotword/toggleOn", { siteId: siteId });
        }
    }, {
        key: 'sendHotwordDetected',

        /**
         * Send Mqtt message to fake hotword detection
         */
        value: function sendHotwordDetected(siteId, hotwordId) {
            this.sendMqtt("hermes/hotword/" + hotwordId + "/detected", { siteId: siteId, modelId: hotwordId, modelType: 'universal' });
        }

        /**
         * Send Mqtt message to start a voice session
         */

    }, {
        key: 'sendStartSession',
        value: function sendStartSession(siteId, customData, text) {
            var payload = { siteId: siteId, init: { type: 'action', canBeEnqueued: false, sendIntentNotRecognized: true } };
            if (customData) payload.customData = JSON.stringify(customData);
            if (text && text.length > 0) payload.init.text = text;
            this.sendMqtt("hermes/dialogueManager/startSession", payload);
        }
    }, {
        key: 'sendSayFinished',


        /**
         * Send Mqtt message to indicate that tts has finished
         */
        value: function sendSayFinished(id, sessionId) {
            this.sendMqtt("hermes/tts/sayFinished", { id: id, sessionId: sessionId });
        }
    }, {
        key: 'sendPlayFinished',


        /**
         * Send Mqtt message to indicate audioserver playback has finished
         */
        value: function sendPlayFinished(siteId, sessionId) {
            this.sendMqtt("hermes/audioServer/" + this.siteId + "/playFinished", { siteId: siteId, id: sessionId });
        }
    }, {
        key: 'sendEndSession',


        /**
         * Send Mqtt message to end the session immediately
         */
        value: function sendEndSession(sessionId, text) {
            var payload = { sessionId: sessionId };
            if (text && text.length > 0) {
                payload.text = text;
            }
            this.sendMqtt("hermes/dialogueManager/endSession", payload);
        }
    }, {
        key: 'sendAsrToggleOn',


        /**
          * Send Mqtt message to toggle on asr
          * Used to forcibly initialise the local asr server.
          */
        value: function sendAsrToggleOn(siteId) {
            this.sendMqtt("hermes/asr/toggleOn", {});
        }
    }, {
        key: 'sendAsrStopListening',


        /**
         * Send Mqtt message to toggle on asr
         * Used to forcibly initialise the local asr server.
         */
        value: function sendAsrStopListening(siteId) {
            this.sendMqtt("hermes/asr/stopListening", { siteId: siteId });
        }
    }, {
        key: 'sendAsrStartListening',


        /**
         * Send Mqtt message to toggle on asr
         * Used to forcibly initialise the local asr server.
         */
        value: function sendAsrStartListening(siteId) {
            this.sendMqtt("hermes/asr/startListening", { siteId: siteId });
        }
    }, {
        key: 'sendAsrToggleOff',


        /**
        * Send Mqtt message to toggle on asr
        * Used to forcibly initialise the local asr server.
        */
        value: function sendAsrToggleOff(siteId) {
            this.sendMqtt("hermes/asr/toggleOff", {});
        }
    }, {
        key: 'sendFeedbackToggleOn',


        /**
        * Send Mqtt message to toggle on feedback
        */
        value: function sendFeedbackToggleOn(siteId) {
            this.sendMqtt("hermes/feedback/sound/toggleOn", { siteId: siteId });
        }
    }, {
        key: 'sendFeedbackToggleOff',


        /**
        * Send Mqtt message to toggle off feedback
        */
        value: function sendFeedbackToggleOff(siteId) {
            this.sendMqtt("hermes/feedback/sound/toggleOff", { siteId: siteId });
        }
    }, {
        key: 'render',
        value: function render() {
            return React.createElement('b', { id: 'snipsreactcomponent' });
        }
    }]);
    return SnipsReactComponent;
}(Component);

//let hark = require('hark');

var SnipsReactMicrophone = function (_SnipsReactComponent) {
    inherits(SnipsReactMicrophone, _SnipsReactComponent);

    function SnipsReactMicrophone(props) {
        classCallCheck(this, SnipsReactMicrophone);

        var _this = possibleConstructorReturn(this, (SnipsReactMicrophone.__proto__ || Object.getPrototypeOf(SnipsReactMicrophone)).call(this, props));

        _this.startRecording = function () {
            this.setState({ lastIntent: '', lastTts: '', lastTranscript: '', showMessage: false });
            this.sendStartSession(this.siteId, { startedBy: 'snipsreactmicrophone', user: this.props.user ? this.props.user._id : '' });
        };

        _this.stopRecording = function () {
            var session = this.logger.getSession(this.siteId, null);
            if (session) this.sendEndSession(session.sessionId);
        };

        var that = _this;
        _this.state = {};
        if (!props.siteId || props.siteId.length === 0) {
            throw "Snips Microphone must be configured with a siteId property";
        }
        _this.state = { recording: false, messages: [], lastIntent: '', lastTts: '', lastTranscript: '', showMessage: false, activated: false, speaking: false, showConfig: false, listening: false, sessionId: "" };
        _this.siteId = props.siteId; //? props.siteId : 'browser'+parseInt(Math.random()*100000000,10);
        _this.clientId = props.clientId ? props.clientId : 'client' + parseInt(Math.random() * 100000000, 10);
        _this.hotwordId = props.hotwordId ? props.hotwordId : 'default';
        _this.context = null;
        _this.gainNode = null;
        _this.messageTimeout = null;
        _this.speakingTimeout = null;
        _this.speechEvents = null;
        _this.startRecording = _this.startRecording.bind(_this);
        _this.stopRecording = _this.stopRecording.bind(_this);
        _this.startRecorder = _this.startRecorder.bind(_this);
        _this.sendAudioBuffer = _this.sendAudioBuffer.bind(_this);
        _this.audioBufferToWav = _this.audioBufferToWav.bind(_this);
        _this.reSample = _this.reSample.bind(_this);
        _this.encodeWAV = _this.encodeWAV.bind(_this);
        _this.interleave = _this.interleave.bind(_this);
        _this.flashState = _this.flashState.bind(_this);
        _this.activate = _this.activate.bind(_this);
        _this.deactivate = _this.deactivate.bind(_this);
        _this.showConfig = _this.showConfig.bind(_this);
        _this.showConfigNow = _this.showConfigNow.bind(_this);
        _this.clearConfigTimer = _this.clearConfigTimer.bind(_this);

        _this.waitingForSession = null;
        _this.configTimeout = null;

        var eventFunctions = {
            // SESSION
            'hermes/asr/textCaptured': function hermesAsrTextCaptured(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId && payload.text && payload.text.length > 0) {
                    that.flashState('lastTranscript', payload.text);
                }
            },
            'hermes/tts/say': function hermesTtsSay(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId && payload.text && payload.text.length > 0) {
                    that.flashState('lastTts', payload.text);
                }
            },
            'hermes/hotword/#/toggleOn': function hermesHotwordToggleOn(payload) {
                if (that.props.enableServerHotword) {
                    if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId) {
                        that.setState({ sending: true });
                    }
                }
            },
            'hermes/hotword/#/toggleOff': function hermesHotwordToggleOff(payload) {
                if (that.props.enableServerHotword) {
                    if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId) {
                        that.setState({ sending: false });
                    }
                }
            },
            'hermes/asr/startListening': function hermesAsrStartListening(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId) {
                    that.setState({ sending: true });
                }
            },
            'hermes/asr/stopListening': function hermesAsrStopListening(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId) {
                    that.setState({ sending: false });
                }
            }
        };

        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        return _this;
    }

    /**
     * Lifecycle functions
     */

    /**
     * Activate on mount if user has previously enabled.
     */


    createClass(SnipsReactMicrophone, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.startRecorder();

            var that = this;
            // FORCE START ASR ON THIS SITE BY TOGGLE ON SESSION, WAIT, THEN END SESSION AND RESET LOGS
            that.queueOneOffCallbacks({
                'hermes/dialogueManager/sessionStarted': function hermesDialogueManagerSessionStarted(payload) {
                    if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId) {
                        // clear log
                        //that.queueOneOffCallbacks({
                        //'hermes/feedback/sound/toggleOn' : function(payload) {
                        //if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.siteId ) {
                        //that.logger.reset();
                        //}
                        //}
                        //});
                        setTimeout(function () {
                            that.sendEndSession(payload.sessionId);
                            that.sendFeedbackToggleOn(that.props.siteId);
                        }, 500);
                    }
                }
            });
            setTimeout(function () {
                that.sendFeedbackToggleOff(that.props.siteId);
                that.sendStartSession(that.props.siteId);
            }, 900);
        }

        /** 
         * Functions to enable and disable configuration screen 
         * by default using a debounce to implement click and hold to enable config
         **/

    }, {
        key: 'showConfig',
        value: function showConfig(e) {
            var that = this;
            this.configTimeout = setTimeout(function () {
                // console.log('show now');
                //that.setState({showConfig:true}); 
                if (that.props.showConfig) that.props.showConfig();else that.deactivate();
            }, 1000);
        }
    }, {
        key: 'showConfigNow',
        value: function showConfigNow(e) {
            e.preventDefault();
            e.stopPropagation();
            //this.setState({showConfig:true});
            if (this.props.showConfig) this.props.showConfig();else this.deactivate();
        }
    }, {
        key: 'clearConfigTimer',
        value: function clearConfigTimer() {
            if (this.configTimeout) clearTimeout(this.configTimeout);
        }
    }, {
        key: 'activate',


        /**
         * Connect to mqtt, start the recorder and optionally start listening
         * Triggered by microphone click or hotword when the mic is deactivated
         */
        value: function activate() {
            var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            var that = this;
            localStorage.setItem(this.appendUserId('snipsmicrophone_enabled', this.props.user), 'true');
            if (start) {
                setTimeout(function () {
                    that.startRecording();
                }, 500);
            }
            that.setState({ activated: true, sending: false });
        }

        /**
         * Enable streaming of the audio input stream
         */

    }, {
        key: 'deactivate',


        /**
         * Disable microphone and listeners
         */
        value: function deactivate() {
            localStorage.setItem(this.appendUserId('snipsmicrophone_enabled', this.props.user), 'false');
            this.setState({ activated: false, sending: false });
        }
    }, {
        key: 'startRecorder',


        /**
        * Access the microphone and start streaming mqtt packets
        */
        value: function startRecorder() {
            var that = this;
            if (!navigator.getUserMedia) {
                navigator.getUserMedia = navigator.getUserMedia || navigator.webkitGetUserMedia || navigator.mozGetUserMedia || navigator.msGetUserMedia;
            }
            try {
                if (navigator.getUserMedia) {
                    navigator.getUserMedia({ audio: true }, success, function (e) {
                        console.log(['MIC Error capturing audio.', e]);
                    });
                } else {
                    console.log('MIC getUserMedia not supported in this browser.');
                }
            } catch (e) {
                console.log(e);
            }

            function success(e) {
                var audioContext = window.AudioContext || window.webkitAudioContext;
                var context = new audioContext();
                that.setState({ 'activated': true });
                that.gainNode = context.createGain();
                // initial set volume
                that.gainNode.gain.value = that.props.config.inputvolume > 0 ? that.props.config.inputvolume / 100 : 0.5;
                var audioInput = context.createMediaStreamSource(e);
                var bufferSize = 256;
                var recorder = context.createScriptProcessor(bufferSize, 1, 1);
                recorder.onaudioprocess = function (e) {
                    //!that.logger.connected || 
                    if (!that.state.sending) return;
                    var left = e.inputBuffer.getChannelData(0);
                    that.sendAudioBuffer(e.inputBuffer, context.sampleRate);
                    //console.log('MIC send audio'); //,buffer,that.audioBuffer]);
                };
                if (that.props.addInputGainNode) that.props.addInputGainNode(that.gainNode);
                audioInput.connect(that.gainNode);
                that.gainNode.connect(recorder);
                recorder.connect(context.destination);
            }
        }
    }, {
        key: 'addInputGainNode',
        value: function addInputGainNode(node) {
            this.inputGainNodes.push(node);
        }
    }, {
        key: 'appendUserId',
        value: function appendUserId(text, user) {
            if (user && user._id) {
                return text + "_" + user._id;
            } else {
                return text;
            }
        }
    }, {
        key: 'flashState',


        ///**
        //* Bind silence recognition events to set speaking state
        //*/ 
        //bindSpeakingEvents(audioContext,e) {
        //// console.log(['bindSpeakingEvents'])
        //let that = this;
        //var options = {audioContext:audioContext};
        //options.threshhold = this.getThreshholdFromVolume(this.state.config.silencesensitivity);
        //// bind speaking events care of hark
        //this.speechEvents = hark(e, options);
        //this.speechEvents.on('speaking', function() {
        //if (that.state.config.silencedetection !== "no") {
        ////console.log('speaking');
        //if (that.speakingTimeout) clearTimeout(that.speakingTimeout);
        //that.setState({speaking:true});
        //}
        //});

        //this.speechEvents.on('stopped_speaking', function() {
        //if (that.state.config.silencedetection !== "no") {
        //if (that.speakingTimeout) clearTimeout(that.speakingTimeout);
        //that.speakingTimeout = setTimeout(function() {
        //// console.log('stop speaking');
        //that.setState({speaking:false});
        //},1000);
        //}

        //});            

        //};

        //getThreshholdFromVolume(volume) {
        //return 10 * Math.log((101 - volume )/800);
        //};


        value: function flashState(key, value) {
            var that = this;
            if (this.props.config.enablenotifications !== "no") {
                if (key && key.length > 0 && value && value.length > 0) {
                    if (this.messageTimeOut) clearTimeout(this.messageTimeOut);
                    var newObj = { showMessage: true };
                    if (key === "lastTranscript") {
                        newObj.lastIntent = '';
                        newObj.lastTts = '';
                    }
                    newObj[key] = value;
                    this.setState(newObj);
                    setTimeout(function () {
                        that.setState({ showMessage: false });
                    }, that.props.messageTimeout > 0 ? that.props.messageTimeout : 10000);
                }
            }
        }
    }, {
        key: 'sendAudioBuffer',


        /** WAV encoding functions */
        value: function sendAudioBuffer(buffer, sampleRate) {
            var that = this;
            if (buffer) {
                this.reSample(buffer, 16000, function (result) {
                    var wav = that.audioBufferToWav(result);
                    that.logger.sendAudioMqtt("hermes/audioServer/" + that.siteId + "/audioFrame", wav);
                }, sampleRate);
            }
        }
    }, {
        key: 'convertFloat32ToInt16',
        value: function convertFloat32ToInt16(buffer) {
            var l = buffer.length;
            var buf = new Int16Array(l);
            while (l--) {
                buf[l] = Math.min(1, buffer[l]) * 0x7FFF;
            }
            return buf.buffer;
        }
    }, {
        key: 'flattenArray',
        value: function flattenArray(inChannelBuffer, recordingLength) {
            var channelBuffer = this.convertFloat32ToInt16(inChannelBuffer);

            var result = new Float32Array(recordingLength);
            var offset = 0;
            for (var i = 0; i < channelBuffer.length; i++) {
                var buffer = channelBuffer[i];
                result.set(buffer, offset);
                offset += buffer.length;
            }
            return result;
        }
    }, {
        key: 'reSample',
        value: function reSample(audioBuffer, targetSampleRate, onComplete, sampleRateContext) {
            var sampleRate = !isNaN(sampleRateContext) ? sampleRateContext : 44100;
            var channel = audioBuffer && audioBuffer.numberOfChannels ? audioBuffer.numberOfChannels : 1;
            var samples = audioBuffer.length * targetSampleRate / sampleRate;
            var offlineContext = new OfflineAudioContext(channel, samples, targetSampleRate);
            var bufferSource = offlineContext.createBufferSource();
            bufferSource.buffer = audioBuffer;

            bufferSource.connect(offlineContext.destination);
            bufferSource.start(0);
            offlineContext.startRendering().then(function (renderedBuffer) {
                onComplete(renderedBuffer);
            }).catch(function (e) {
                console.log(e);
            });
        }
    }, {
        key: 'audioBufferToWav',
        value: function audioBufferToWav(buffer, opt) {
            opt = opt || {};

            var numChannels = buffer.numberOfChannels;
            var sampleRate = buffer.sampleRate;
            var format = opt.float32 ? 3 : 1;
            var bitDepth = format === 3 ? 32 : 16;

            var result;
            if (numChannels === 2) {
                result = this.interleave(buffer.getChannelData(0), buffer.getChannelData(1));
            } else {
                result = buffer.getChannelData(0);
            }

            return this.encodeWAV(result, format, sampleRate, numChannels, bitDepth);
        }
    }, {
        key: 'encodeWAV',
        value: function encodeWAV(samples, format, sampleRate, numChannels, bitDepth) {
            var bytesPerSample = bitDepth / 8;
            var blockAlign = numChannels * bytesPerSample;

            var buffer = new ArrayBuffer(44 + samples.length * bytesPerSample);
            var view = new DataView(buffer);

            /* RIFF identifier */
            this.writeString(view, 0, 'RIFF');
            /* RIFF chunk length */
            view.setUint32(4, 36 + samples.length * bytesPerSample, true);
            /* RIFF type */
            this.writeString(view, 8, 'WAVE');
            /* format chunk identifier */
            this.writeString(view, 12, 'fmt ');
            /* format chunk length */
            view.setUint32(16, 16, true);
            /* sample format (raw) */
            view.setUint16(20, format, true);
            /* channel count */
            view.setUint16(22, numChannels, true);
            /* sample rate */
            view.setUint32(24, sampleRate, true);
            /* byte rate (sample rate * block align) */
            view.setUint32(28, sampleRate * blockAlign, true);
            /* block align (channel count * bytes per sample) */
            view.setUint16(32, blockAlign, true);
            /* bits per sample */
            view.setUint16(34, bitDepth, true);
            /* data chunk identifier */
            this.writeString(view, 36, 'data');
            /* data chunk length */
            view.setUint32(40, samples.length * bytesPerSample, true);
            if (format === 1) {
                // Raw PCM
                this.floatTo16BitPCM(view, 44, samples);
            } else {
                this.writeFloat32(view, 44, samples);
            }

            return buffer;
        }
    }, {
        key: 'interleave',
        value: function interleave(inputL, inputR) {
            var length = inputL.length + inputR.length;
            var result = new Float32Array(length);

            var index = 0;
            var inputIndex = 0;

            while (index < length) {
                result[index++] = inputL[inputIndex];
                result[index++] = inputR[inputIndex];
                inputIndex++;
            }
            return result;
        }
    }, {
        key: 'writeFloat32',
        value: function writeFloat32(output, offset, input) {
            for (var i = 0; i < input.length; i++, offset += 4) {
                output.setFloat32(offset, input[i], true);
            }
        }
    }, {
        key: 'floatTo16BitPCM',
        value: function floatTo16BitPCM(output, offset, input) {
            for (var i = 0; i < input.length; i++, offset += 2) {
                var s = Math.max(-1, Math.min(1, input[i]));
                output.setInt16(offset, s < 0 ? s * 0x8000 : s * 0x7FFF, true);
            }
        }
    }, {
        key: 'writeString',
        value: function writeString(view, offset, string) {
            for (var i = 0; i < string.length; i++) {
                view.setUint8(offset + i, string.charCodeAt(i));
            }
        }
    }, {
        key: 'render',
        value: function render() {
            var text = this.props.text;


            var buttonStyle = this.props.buttonStyle ? this.props.buttonStyle : {};

            var size = this.props.size && this.props.size.length > 0 ? this.props.size : '2em';
            buttonStyle.width = size;
            buttonStyle.height = size;

            var position = this.props.position && this.props.position.length > 0 ? this.props.position : 'topright';
            // set mic position and override bubble position
            buttonStyle.position = 'fixed';
            if (position === "topleft") {
                buttonStyle.top = 0;
                buttonStyle.left = 0;
            } else if (position === "topright") {
                buttonStyle.top = 0;
                buttonStyle.right = 0;
            } else if (position === "bottomleft") {
                buttonStyle.bottom = 0;
                buttonStyle.left = 0;
            } else if (position === "bottomright") {
                buttonStyle.bottom = 0;
                buttonStyle.right = 0;
            }

            var status = 0;
            if (this.state.activated) status = 2;
            //if (this.state.connected) status = 2;
            if (this.state.sending) status = 3;
            //if (this.state.sending) status = 3;
            // console.log(status);
            var borderColor = 'black';
            var borderWidth = 2;
            if (status == 3) {
                borderColor = this.state.speaking ? 'darkgreen' : this.props.borderColor ? this.props.borderColor : 'green';
                buttonStyle.backgroundColor = 'lightgreen';
                if (this.state.speaking) borderWidth = 3;
            } else if (status == 1) {
                borderColor = this.state.speaking ? 'darkorange' : this.props.borderColor ? this.props.borderColor : 'orange';
                buttonStyle.backgroundColor = 'lightorange';
                if (this.state.speaking) borderWidth = 3;
            } else if (status == 2) {
                borderColor = this.state.speaking ? 'orangered' : this.props.borderColor ? this.props.borderColor : 'red';
                buttonStyle.backgroundColor = 'pink';
                if (this.state.speaking) borderWidth = 3;
            } else {
                borderColor = this.props.borderColor ? this.props.borderColor : 'black';
                buttonStyle.backgroundColor = 'lightgrey';
            }
            if (!buttonStyle.padding) buttonStyle.padding = '0.5em';
            if (!buttonStyle.margin) buttonStyle.margin = '0.5em';
            // console.log(borderColor);
            //if (!buttonStyle.border) 
            buttonStyle.border = borderWidth + 'px solid ' + borderColor;
            if (!buttonStyle.borderRadius) buttonStyle.borderRadius = '100px';
            // console.log(['BUTTON STYLE',buttonStyle]);

            var micOffIcon = React.createElement(
                'svg',
                { style: buttonStyle, 'aria-hidden': 'true', 'data-prefix': 'fas', 'data-icon': 'microphone', className: 'svg-inline--fa fa-microphone fa-w-11', role: 'img', xmlns: 'http://www.w3.org/2000/svg', viewBox: '0 0 352 512' },
                React.createElement('path', { fill: 'currentColor', d: 'M176 352c53.02 0 96-42.98 96-96V96c0-53.02-42.98-96-96-96S80 42.98 80 96v160c0 53.02 42.98 96 96 96zm160-160h-16c-8.84 0-16 7.16-16 16v48c0 74.8-64.49 134.82-140.79 127.38C96.71 376.89 48 317.11 48 250.3V208c0-8.84-7.16-16-16-16H16c-8.84 0-16 7.16-16 16v40.16c0 89.64 63.97 169.55 152 181.69V464H96c-8.84 0-16 7.16-16 16v16c0 8.84 7.16 16 16 16h160c8.84 0 16-7.16 16-16v-16c0-8.84-7.16-16-16-16h-56v-33.77C285.71 418.47 352 344.9 352 256v-48c0-8.84-7.16-16-16-16z' })
            );

            var micOnIcon = React.createElement(
                'svg',
                { style: buttonStyle, 'aria-hidden': 'true', 'data-prefix': 'fas', 'data-icon': 'microphone-slash', className: 'svg-inline--fa fa-microphone-slash fa-w-20', role: 'img', xmlns: 'http://www.w3.org/2000/svg', viewBox: '0 0 640 512' },
                React.createElement('path', { fill: 'currentColor', d: 'M633.82 458.1l-157.8-121.96C488.61 312.13 496 285.01 496 256v-48c0-8.84-7.16-16-16-16h-16c-8.84 0-16 7.16-16 16v48c0 17.92-3.96 34.8-10.72 50.2l-26.55-20.52c3.1-9.4 5.28-19.22 5.28-29.67V96c0-53.02-42.98-96-96-96s-96 42.98-96 96v45.36L45.47 3.37C38.49-2.05 28.43-.8 23.01 6.18L3.37 31.45C-2.05 38.42-.8 48.47 6.18 53.9l588.36 454.73c6.98 5.43 17.03 4.17 22.46-2.81l19.64-25.27c5.41-6.97 4.16-17.02-2.82-22.45zM400 464h-56v-33.77c11.66-1.6 22.85-4.54 33.67-8.31l-50.11-38.73c-6.71.4-13.41.87-20.35.2-55.85-5.45-98.74-48.63-111.18-101.85L144 241.31v6.85c0 89.64 63.97 169.55 152 181.69V464h-56c-8.84 0-16 7.16-16 16v16c0 8.84 7.16 16 16 16h160c8.84 0 16-7.16 16-16v-16c0-8.84-7.16-16-16-16z' })
            );

            var resetIcon = React.createElement(
                'svg',
                { 'aria-hidden': 'true', style: { height: '1.1em' }, role: 'img', xmlns: 'http://www.w3.org/2000/svg', viewBox: '0 0 512 512' },
                React.createElement('path', { fill: 'currentColor', d: 'M500.333 0h-47.411c-6.853 0-12.314 5.729-11.986 12.574l3.966 82.759C399.416 41.899 331.672 8 256.001 8 119.34 8 7.899 119.526 8 256.187 8.101 393.068 119.096 504 256 504c63.926 0 122.202-24.187 166.178-63.908 5.113-4.618 5.354-12.561.482-17.433l-33.971-33.971c-4.466-4.466-11.64-4.717-16.38-.543C341.308 415.448 300.606 432 256 432c-97.267 0-176-78.716-176-176 0-97.267 78.716-176 176-176 60.892 0 114.506 30.858 146.099 77.8l-101.525-4.865c-6.845-.328-12.574 5.133-12.574 11.986v47.411c0 6.627 5.373 12 12 12h200.333c6.627 0 12-5.373 12-12V12c0-6.627-5.373-12-12-12z' })
            );

            var stopIcon2 = React.createElement(
                'svg',
                { 'aria-hidden': 'true', style: { height: '1.4em' }, role: 'img', xmlns: 'http://www.w3.org/2000/svg', viewBox: '0 0 512 512' },
                React.createElement('path', { fill: 'currentColor', d: 'M440.5 88.5l-52 52L415 167c9.4 9.4 9.4 24.6 0 33.9l-17.4 17.4c11.8 26.1 18.4 55.1 18.4 85.6 0 114.9-93.1 208-208 208S0 418.9 0 304 93.1 96 208 96c30.5 0 59.5 6.6 85.6 18.4L311 97c9.4-9.4 24.6-9.4 33.9 0l26.5 26.5 52-52 17.1 17zM500 60h-24c-6.6 0-12 5.4-12 12s5.4 12 12 12h24c6.6 0 12-5.4 12-12s-5.4-12-12-12zM440 0c-6.6 0-12 5.4-12 12v24c0 6.6 5.4 12 12 12s12-5.4 12-12V12c0-6.6-5.4-12-12-12zm33.9 55l17-17c4.7-4.7 4.7-12.3 0-17-4.7-4.7-12.3-4.7-17 0l-17 17c-4.7 4.7-4.7 12.3 0 17 4.8 4.7 12.4 4.7 17 0zm-67.8 0c4.7 4.7 12.3 4.7 17 0 4.7-4.7 4.7-12.3 0-17l-17-17c-4.7-4.7-12.3-4.7-17 0-4.7 4.7-4.7 12.3 0 17l17 17zm67.8 34c-4.7-4.7-12.3-4.7-17 0-4.7 4.7-4.7 12.3 0 17l17 17c4.7 4.7 12.3 4.7 17 0 4.7-4.7 4.7-12.3 0-17l-17-17zM112 272c0-35.3 28.7-64 64-64 8.8 0 16-7.2 16-16s-7.2-16-16-16c-52.9 0-96 43.1-96 96 0 8.8 7.2 16 16 16s16-7.2 16-16z' })
            );
            var config = this.props.config;
            return React.createElement(
                'div',
                { id: 'snipsreactmicrophone', style: { zIndex: '9999' } },
                !this.state.activated && React.createElement(
                    'span',
                    { onClick: this.activate },
                    micOnIcon
                ),
                this.state.activated && this.state.sending && React.createElement(
                    'span',
                    { onTouchStart: this.showConfig, onTouchEnd: this.clearConfigTimer, onMouseDown: this.showConfig, onMouseUp: this.clearConfigTimer, onContextMenu: this.showConfigNow, onClick: this.stopRecording },
                    micOffIcon
                ),
                this.state.activated && !this.state.sending && React.createElement(
                    'span',
                    { onTouchStart: this.showConfig, onTouchEnd: this.clearConfigTimer, onMouseDown: this.showConfig, onMouseUp: this.clearConfigTimer, onContextMenu: this.showConfigNow, onClick: this.startRecording },
                    micOnIcon
                ),
                this.state.showMessage && React.createElement(
                    'div',
                    { style: { padding: '1em', borderRadius: '20px', backgroundColor: 'skyblue', margin: '5%', width: '90%', top: '1.7em', color: 'black', border: '2px solid blue', zIndex: '9999' } },
                    this.state.lastTranscript && React.createElement(
                        'div',
                        { style: { fontStyle: 'italic' } },
                        this.state.lastTranscript
                    ),
                    false && this.state.lastIntent && React.createElement(
                        'div',
                        null,
                        this.state.lastIntent
                    ),
                    this.state.lastTts && React.createElement(
                        'div',
                        null,
                        this.state.lastTts
                    )
                ),
                React.createElement('div', { id: 'audio' })
            );
        }
    }]);
    return SnipsReactMicrophone;
}(SnipsReactComponent);

var SnipsReactSpeaker = function (_SnipsReactComponent) {
    inherits(SnipsReactSpeaker, _SnipsReactComponent);

    function SnipsReactSpeaker(props) {
        classCallCheck(this, SnipsReactSpeaker);

        var _this = possibleConstructorReturn(this, (SnipsReactSpeaker.__proto__ || Object.getPrototypeOf(SnipsReactSpeaker)).call(this, props));

        var that = _this;

        if (!props.siteId || props.siteId.length === 0) {
            throw "Speaker must be configured with a siteId property";
        }
        _this.playSound = _this.playSound.bind(_this);
        _this.setVolume = _this.setVolume.bind(_this);
        _this.state = { volume: .5 };

        var eventFunctions = {
            // SESSION
            'hermes/audioServer/#/playBytes': function hermesAudioServerPlayBytes(destination, siteId, id, session, audio) {
                if (siteId && siteId.length > 0 && siteId === that.props.siteId) {
                    that.playSound(audio).then(function () {
                        //// TODO wait for mqtt mesage    
                        that.sendMqtt("hermes/audioServer/" + siteId + "/playFinished", { id: id, siteId: siteId, sessionId: session ? session.sessionId : null });
                    });
                }
            },
            'hermes/hotword/#/detected': function hermesHotwordDetected(payload) {
                // quarter volume for 10 seconds
            }
        };

        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        return _this;
    }

    createClass(SnipsReactSpeaker, [{
        key: 'setVolume',
        value: function setVolume(volume) {
            // console.log(['SET VOLUME']);
            this.setState({ volume: volume });
            //this.gainNode.gain.value = this.state.volume;
        }
    }, {
        key: 'playSound',
        value: function playSound(bytes) {
            var that = this;
            return new Promise(function (resolve, reject) {
                if (that.props.config.enableaudio !== "no") {
                    var buffer = new Uint8Array(bytes.length);
                    buffer.set(new Uint8Array(bytes), 0);
                    var audioContext = window.AudioContext || window.webkitAudioContext;
                    var context = new audioContext();
                    var gainNode = context.createGain();
                    // initial set volume
                    gainNode.gain.value = that.props.config.outputvolume / 100 ? that.props.config.outputvolume / 100 : 0.5;
                    context.decodeAudioData(buffer.buffer, function (audioBuffer) {
                        var source = context.createBufferSource();
                        source.buffer = audioBuffer;
                        source.connect(gainNode);
                        gainNode.connect(context.destination);
                        source.start(0);
                        source.onended = function () {
                            resolve();
                        };
                    });
                } else {
                    resolve();
                }
            });
        }
    }, {
        key: 'render',
        value: function render() {
            return React.createElement('span', { id: 'snipsreactspeaker' });
        }
    }]);
    return SnipsReactSpeaker;
}(SnipsReactComponent);

/* global window */

var SnipsAppServer = function (_SnipsReactComponent) {
    inherits(SnipsAppServer, _SnipsReactComponent);

    function SnipsAppServer(props) {
        classCallCheck(this, SnipsAppServer);

        var _this = possibleConstructorReturn(this, (SnipsAppServer.__proto__ || Object.getPrototypeOf(SnipsAppServer)).call(this, props));

        var that = _this;
        _this.state = {};
        var eventFunctions = {
            // SESSION
            'hermes/intent/#': function hermesIntent(payload, message) {
                //console.log(['INTENT',this,payload,message]);
                var parts = message.destinationName ? message.destinationName.split("/") : [];
                if (parts.length > 0 && parts[0] === "hermes") {
                    if (parts.length > 1 && parts[1] === "intent") {
                        var _payload = {};
                        var intent = parts[2];
                        if (intent && intent.length > 0) {
                            try {
                                _payload = JSON.parse(message.payloadString);
                                if (that.props.intents && that.props.intents.hasOwnProperty(intent) && that.props.intents[intent]) {
                                    var p = that.props.intents[intent].bind(that)(_payload);
                                    if (p && p.then) {
                                        p.then(function (v) {
                                            //console.log(['APP SERVER MESSAGE SUCCESS',intent,payload,v]);
                                            that.sendEndSession.bind(that)(_payload.sessionId);
                                        }).catch(function (v) {
                                            //console.log(['APP SERVER MESSAGE REJECT',intent,payload,v]);
                                            that.sendEndSession.bind(that)(_payload.sessionId);
                                        });
                                    } else {
                                        //console.log(['APP SERVER MESSAGE SUCCESS no promise',intent,payload,p]);
                                        that.sendEndSession.bind(that)(_payload.sessionId);
                                    }
                                } else {
                                    //console.log(['APP SERVER MESSAGE no intent',intent,payload]);
                                    that.sendEndSession.bind(that)(_payload.sessionId);
                                }
                            } catch (e) {
                                //console.log(['APP SERVER FAILED TO PARSE PAYLOAD',e]);
                                //that.sendEndSession.bind(that)();
                            }
                        }
                    }
                }
            }
        };

        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        return _this;
    }

    createClass(SnipsAppServer, [{
        key: 'cleanSlots',
        value: function cleanSlots(slots) {
            var final = {};
            if (slots) {
                slots.map(function (slot) {
                    final[slot.slotName] = { type: slot.value.kind, value: slot.value.value };
                    return;
                });
            }
            return final;
        }
    }, {
        key: 'render',
        value: function render() {
            return React.createElement('span', { id: 'snipsreactappserver' });
        }
    }]);
    return SnipsAppServer;
}(SnipsReactComponent);

var Resources = {};
Resources.keywordIDs = {
    'ok lamp': new Uint8Array([0xac, 0x24, 0x75, 0x21, 0x14, 0x3d, 0x2a, 0xe7, 0x0a, 0x85, 0x75, 0x4c, 0x48, 0x31, 0x5b, 0x44, 0x4b, 0xb6, 0xe8, 0xc3, 0x77, 0x30, 0xd5, 0xac, 0xca, 0x54, 0x06, 0x29, 0xbd, 0x15, 0xca, 0x90, 0x55, 0x81, 0xae, 0x21, 0x6a, 0x04, 0x1e, 0x5a, 0x9d, 0x64, 0x83, 0x0c, 0x04, 0x03, 0x6b, 0xe8, 0x22, 0x2e, 0x19, 0xbf, 0x7e, 0x2b, 0x4d, 0x8c, 0x50, 0x27, 0xb6, 0x11, 0xf3, 0x17, 0xc3, 0xf9, 0xe3, 0x69, 0x19, 0x26, 0xbe, 0x0d, 0xad, 0x78, 0x74, 0x61, 0x4b, 0xb8, 0xde, 0x83, 0x1c, 0xb9, 0xa1, 0x06, 0x27, 0x77, 0x03, 0xb2, 0x24, 0x82]),
    'navy blue': new Uint8Array([0xd9, 0x36, 0xb2, 0xcc, 0x5d, 0xbb, 0x2b, 0x66, 0xae, 0xbb, 0x39, 0xf3, 0x24, 0xf4, 0x02, 0xf2, 0xb9, 0x5a, 0xf7, 0xd7, 0x8d, 0x02, 0xbc, 0x7b, 0xa3, 0x04, 0xb3, 0xfd, 0x2c, 0x0b, 0x9c, 0x10, 0x2c, 0x28, 0x6f, 0x65, 0x3f, 0xb9, 0x39, 0x08, 0x44, 0x62, 0x47, 0x3a, 0xd8, 0x6d, 0xe7, 0x4a, 0xd9, 0x64, 0x50, 0x6b, 0xd0, 0x39, 0x7e, 0x43, 0x05, 0xeb, 0xf8, 0xc2, 0xe7, 0xab, 0xf5, 0x39, 0x88, 0xd4, 0x99, 0x3c, 0x2d, 0x2a, 0xf2, 0xeb, 0x69, 0x5b, 0x34, 0x7b, 0x51, 0x61, 0x83, 0x82, 0x08, 0x76, 0xdf, 0x86, 0xab, 0xe8, 0x83, 0x48])
    //'yellow': new Uint8Array([
    //0x6f, 0x7d, 0x2c, 0x10, 0xcb, 0xf0, 0x76, 0x1b, 0xee, 0xeb, 0x80, 0xaa,
    //0xe1, 0xa5, 0xfa, 0x53, 0xe3, 0xce, 0x4b, 0x59, 0x7d, 0xeb, 0xe3, 0x11,
    //0x13, 0x87, 0x61, 0xa5, 0xef, 0x04, 0x04, 0x37, 0x63, 0xf6, 0x63, 0x91,
    //0x44, 0x22, 0xf2, 0x50, 0xab, 0xb7, 0x35, 0x8c, 0xcf, 0x33, 0xf7, 0xc1,
    //0xf1, 0x06, 0x6c, 0x37, 0x6b, 0x2d, 0x50, 0x0e, 0x1c, 0x1c, 0xee, 0x9d,
    //0x3d, 0x56, 0xa6, 0x54, 0x51, 0xb2, 0xf4, 0x3d, 0x76, 0x7b, 0xcf, 0x4c]),
    //'orange': new Uint8Array([
    //0xb6, 0x3e, 0x3e, 0xbf, 0x78, 0x5f, 0x1c, 0xc4, 0xf2, 0xdb, 0x0e, 0xca,
    //0xd6, 0xb2, 0xad, 0x34, 0x35, 0xe7, 0x90, 0xfb, 0xde, 0x7f, 0x8e, 0x99,
    //0xab, 0xd1, 0x6b, 0x55, 0x71, 0x94, 0x5c, 0x12, 0xd4, 0x81, 0x1d, 0xd4,
    //0x71, 0xb2, 0x2b, 0x11, 0x1d, 0xe6, 0x52, 0xdb, 0x30, 0x44, 0x0c, 0xf9,
    //0xb0, 0xc2, 0x97, 0x3a, 0xe5, 0x78, 0x72, 0xd2, 0x24, 0xd3, 0xde, 0x4e,
    //0xb0, 0xd5, 0xb9, 0xdf, 0x77, 0xd4, 0x2f, 0x2b, 0x80, 0x1a, 0xe5, 0x5c,
    //0x8c, 0x9e, 0xc5, 0xb3]),
    //'purple': new Uint8Array([
    //0xae, 0xce, 0x95, 0x72, 0x5d, 0x85, 0x46, 0x3b, 0x45, 0x50, 0x72, 0xb3,
    //0xd1, 0x80, 0xfa, 0x1c, 0x17, 0x4f, 0x6c, 0x2a, 0x76, 0x5e, 0x0e, 0xb0,
    //0x2d, 0x1b, 0x62, 0x8d, 0xa3, 0xf5, 0x64, 0xff, 0xc7, 0xf6, 0xf2, 0xe3,
    //0xc5, 0x34, 0xef, 0x78, 0x1e, 0xb1, 0xeb, 0x0c, 0xaa, 0x29, 0x5b, 0xb3,
    //0xa8, 0x25, 0x14, 0xd4, 0x80, 0xbf, 0x93, 0x89, 0xe1, 0x7c, 0x5e, 0xd8,
    //0x08, 0x3a, 0xbf, 0x9f, 0x7f, 0xe7, 0x79, 0x13, 0xed, 0xbc, 0xa3, 0x0b,
    //0xf4, 0xd2, 0xfc, 0xf2]),
    //'white': new Uint8Array([
    //0x5e, 0x52, 0xf1, 0xab, 0x90, 0xae, 0x36, 0x5f, 0x98, 0xd2, 0x96, 0xba,
    //0xd8, 0x13, 0x16, 0xf1, 0x66, 0x73, 0xa5, 0xbb, 0x95, 0x0d, 0xfa, 0x9e,
    //0x47, 0x05, 0xc7, 0x3d, 0x36, 0xaf, 0x97, 0x14, 0x98, 0xbc, 0x15, 0xfb,
    //0xa0, 0xdc, 0xb1, 0x60, 0xa2, 0x60, 0x1d, 0x0a, 0x66, 0x5f, 0x47, 0xb8,
    //0xfd, 0x71, 0x3b, 0xe3, 0x06, 0x8d, 0xf4, 0xbd, 0xc6, 0x02, 0xbe, 0x47,
    //0xde, 0x7f, 0xa4, 0x12, 0x9f, 0x07, 0x93, 0xc4, 0xc7, 0x97, 0xd8, 0x9a])

};

/* global window */

var SnipsReactHotwordServer = function (_SnipsReactComponent) {
    inherits(SnipsReactHotwordServer, _SnipsReactComponent);

    function SnipsReactHotwordServer(props) {
        classCallCheck(this, SnipsReactHotwordServer);

        var _this = possibleConstructorReturn(this, (SnipsReactHotwordServer.__proto__ || Object.getPrototypeOf(SnipsReactHotwordServer)).call(this, props));

        _this.sensitivities = new Float32Array([1]);
        _this.hotwordManager = null;
        if (!props.siteId || props.siteId.length === 0) {
            throw "HOTWORD Server must be configured with a siteId property";
        }
        var that = _this;

        _this.gainNode = null;
        _this.hotwordId = _this.props.hotwordId && _this.props.hotwordId.length > 0 ? _this.props.hotwordId : 'default';

        _this.sendHotwordDetected = _this.sendHotwordDetected.bind(_this);
        _this.sendHotwordToggleOn = _this.sendHotwordToggleOn.bind(_this);

        _this.startHotword = _this.startHotword.bind(_this);
        _this.stopHotword = _this.stopHotword.bind(_this);
        _this.hotwordCallback = _this.hotwordCallback.bind(_this);
        var eventFunctions = {
            // SESSION
            'hermes/hotword/toggleOn': function hermesHotwordToggleOn(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.props.siteId) {
                    that.startHotword(that.props.siteId);
                }
            },
            'hermes/hotword/toggleOff': function hermesHotwordToggleOff(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === that.props.siteId) {
                    that.stopHotword();
                }
            }
        };
        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        return _this;
    }

    createClass(SnipsReactHotwordServer, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            var that = this;
            if (this.props.config.hotword.startsWith("browser:")) {
                setTimeout(function () {
                    that.startHotword(that.props.siteId);
                }, 1000);
            }
        }
    }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(props, state) {
            //  console.log(['HW DID UPDATE',props,state,JSON.parse(JSON.stringify(this.props)),this.props.config.hotword + ':' + props.config.hotword]);
            var that = this;
            if (props.inputvolume != this.props.inputvolume) ;
            if (props.config.hotword != this.props.config.hotword) {
                this.hotwordManager = null;
                // console.log(['RESTART HOTWORD']);
                if (this.props.config.hotword.startsWith("browser:")) {
                    setTimeout(function () {
                        that.startHotword(that.props.siteId);
                    }, 1000);
                }
            }
            return false;
        }
    }, {
        key: 'stopHotword',


        /**
         * Pause the hotword manager
         */
        value: function stopHotword() {
            if (this.hotwordManager) this.hotwordManager.pauseProcessing();
        }
    }, {
        key: 'startHotword',


        /**
         * Create or continue the hotword manager
         */
        value: function startHotword(siteId) {
            if (siteId === this.props.siteId) {
                if (this.hotwordManager === null) {
                    //console.log(['REALLY START HOTWORD',this.props.config.hotword]);
                    var parts = this.props.config.hotword.split(":");
                    if (parts.length > 1) {
                        var localHotword = parts[1];
                        this.hotwordManager = new PicovoiceAudioManager(this.props.addInputGainNode, this.props.config.inputvolume);
                        var singleSensitivity = this.props.config.hotwordsensitivity ? this.props.config.hotwordsensitivity / 100 : 0.9;
                        var sensitivities = new Float32Array([singleSensitivity]);
                        var selectedKeyword = null;
                        if (Resources.keywordIDs.hasOwnProperty(localHotword)) {
                            selectedKeyword = Resources.keywordIDs[localHotword];
                            //  console.log(['SELECTED KW',localHotword,selectedKeyword]);
                            this.hotwordManager.start(Porcupine.create([selectedKeyword], sensitivities), this.hotwordCallback, function (e) {
                                console.log(['HOTWORD error', e]);
                            });
                        }
                    }
                } else {
                    if (this.hotwordManager) this.hotwordManager.continueProcessing();
                }
            }
        }
    }, {
        key: 'hotwordCallback',
        value: function hotwordCallback(value) {
            if (!isNaN(value) && parseInt(value, 10) >= 0) {
                this.sendStartSession(this.props.siteId, { startedBy: 'snipsreacthotword', user: this.props.user ? this.props.user._id : '' });
            }
        }
    }, {
        key: 'render',
        value: function render() {
            return React.createElement('span', { id: 'snipsreacthotwordserver' });
        }
    }]);
    return SnipsReactHotwordServer;
}(SnipsReactComponent);

var SnipsReactTts = function (_SnipsReactComponent) {
    inherits(SnipsReactTts, _SnipsReactComponent);

    function SnipsReactTts(props) {
        classCallCheck(this, SnipsReactTts);

        var _this = possibleConstructorReturn(this, (SnipsReactTts.__proto__ || Object.getPrototypeOf(SnipsReactTts)).call(this, props));

        if (!props.siteId || props.siteId.length === 0) {
            throw "TTS Server must be configured with a siteId property";
        }
        //this.props.config={}
        var that = _this;
        var eventFunctions = {
            // SESSION
            'hermes/tts/say': function hermesTtsSay(payload) {
                if (payload.siteId && payload.siteId.length > 0 && payload.siteId === props.siteId) {
                    if (payload.text && payload.text.length > 0) {
                        that.say(payload.text);
                    }
                    that.sendMqtt('hermes/tts/sayFinished', { id: payload.id, sessionId: payload.sessionId });
                }
            }
        };

        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        return _this;
    }

    createClass(SnipsReactTts, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.initSpeechSynthesis();
        }
    }, {
        key: 'say',


        /**
          * Synthesise speech from text and send to to audio output
          */
        value: function say(text) {
            console.log(['TTS SAY', this.props, this.props.config, this.props.config.enablevoice]);
            if (this.props.config.enabletts !== "no") {
                var voice = this.props.config && this.props.config.ttsvoice ? this.props.config.ttsvoice : 'default';

                if (voice === "default") {
                    // js generated fallback
                    speak(text, {
                        amplitude: !isNaN(parseFloat(this.props.config.voicevolume)) ? parseFloat(this.props.config.voicevolume) : 70,
                        pitch: !isNaN(parseFloat(this.props.config.voicepitch)) ? parseFloat(this.props.config.voicepitch) : 50,
                        speed: !isNaN(parseFloat(this.props.config.voicerate)) ? parseFloat(this.props.config.voicerate) * 2.2 : 175
                    });
                } else {
                    // Create a new instance of SpeechSynthesisUtterance.
                    var msg = new SpeechSynthesisUtterance();
                    msg.text = text;
                    msg.volume = !isNaN(parseFloat(this.props.config.voicevolume)) ? parseFloat(this.props.config.voicevolume) : 50;
                    msg.rate = !isNaN(parseFloat(this.props.config.voicerate)) ? parseFloat(this.props.config.voicerate) / 100 : 50 / 100;
                    msg.pitch = !isNaN(parseFloat(this.props.config.voicepitch)) ? parseFloat(this.props.config.voicepitch) : 50;
                    var voices = speechSynthesis.getVoices();

                    // Loop through each of the voices.
                    voices.forEach(function (voiceItem, i) {
                        if (voiceItem.name === voice) msg.voice = voiceItem;
                        window.speechSynthesis.speak(msg);
                    });
                }
            }
        }
    }, {
        key: 'initSpeechSynthesis',
        value: function initSpeechSynthesis() {
            var that = this;
            if ('speechSynthesis' in window) {
                // Fetch the list of voices and populate the voice options.
                var loadVoices = function loadVoices() {
                    // Fetch the available voices.
                    var voices = speechSynthesis.getVoices();

                    // Loop through each of the voices
                    var voiceOptions = [];
                    voices.forEach(function (voice, i) {
                        // Create a new option element.
                        voiceOptions.push({ 'name': voice.name, label: voice.name });
                    });
                    voiceOptions.push({ 'name': 'default', label: 'Browser Generated' });
                    that.setState({ voices: voiceOptions });
                    //console.log(['VOICES a',voiceOptions]);
                };

                // Execute loadVoices.


                loadVoices();

                // Chrome loads voices asynchronously.
                window.speechSynthesis.onvoiceschanged = function (e) {
                    loadVoices();
                };
            } else {
                var voiceOptions = [];
                voiceOptions.push({ 'name': 'default', label: 'Browser Generated' });
                that.setState({ voices: voiceOptions });
            }
            // console.log(['LOADED VOICES',this.state.voices]);
        }
    }, {
        key: 'render',
        value: function render() {
            return React.createElement('b', { id: 'snipsreacttts' });
        }
    }]);
    return SnipsReactTts;
}(SnipsReactComponent);

/* global window */

var SnipsReactLogger = function (_Component) {
    inherits(SnipsReactLogger, _Component);

    function SnipsReactLogger(props) {
        classCallCheck(this, SnipsReactLogger);

        var _this = possibleConstructorReturn(this, (SnipsReactLogger.__proto__ || Object.getPrototypeOf(SnipsReactLogger)).call(this, props));

        _this.state = { showLogMessages: {}, showLog: {} };
        _this.toggleMessageExpansion = _this.toggleMessageExpansion.bind(_this);
        _this.isLogMessageExpanded = _this.isLogMessageExpanded.bind(_this);
        _this.isLogExpanded = _this.isLogExpanded.bind(_this);
        _this.toggleLogExpansion = _this.toggleLogExpansion.bind(_this);
        return _this;
    }

    createClass(SnipsReactLogger, [{
        key: 'toggleMessageExpansion',
        value: function toggleMessageExpansion(e, key) {
            var showLogMessages = this.state.showLogMessages;
            if (this.isLogMessageExpanded(key)) {
                showLogMessages[key] = false;
            } else {
                showLogMessages[key] = true;
            }
            this.setState({ showLogMessages: showLogMessages });
        }
    }, {
        key: 'toggleLogExpansion',
        value: function toggleLogExpansion(e, key) {
            var showLog = this.state.showLog;
            if (this.isLogExpanded(key)) {
                showLog[key] = false;
            } else {
                showLog[key] = true;
            }
            this.setState({ showLog: showLog });
        }
    }, {
        key: 'isLogMessageExpanded',
        value: function isLogMessageExpanded(key) {
            if (this.state.showLogMessages.hasOwnProperty(key) && this.state.showLogMessages[key]) {
                return true;
            }
            return false;
        }
    }, {
        key: 'isLogExpanded',
        value: function isLogExpanded(key) {
            if (this.state.showLog.hasOwnProperty(key) && this.state.showLog[key]) {
                return true;
            }
            return false;
        }
    }, {
        key: 'render',
        value: function render() {
            var that = this;
            if (this.props.sites) {
                var sitesRendered = Object.keys(this.props.sites).map(function (siteKeyIn) {
                    var siteKey = siteKeyIn && siteKeyIn.length > 0 ? siteKeyIn : 'unknownSite';
                    var site = that.props.sites[siteKey];
                    var sessions = Object.values(site);
                    sessions.sort(function (a, b) {
                        if (a.starttimestamp < b.starttimestamp) return 1;else return -1;
                    });
                    if (!that.props.siteId || that.props.siteId && siteKey === that.props.siteId) {
                        var sessionsRendered = sessions.map(function (session, sessionLoopKey) {
                            if (session) {
                                var logs = that.props.messages.map(function (val, key) {
                                    if (val.sessionId === session.sessionId) {
                                        return React.createElement(
                                            'div',
                                            { key: key },
                                            React.createElement(
                                                'button',
                                                { onClick: function onClick(e) {
                                                        return that.toggleMessageExpansion(e, key);
                                                    } },
                                                '+'
                                            ),
                                            '\xA0\xA0',
                                            val.text,
                                            that.isLogMessageExpanded(key) && React.createElement(
                                                'pre',
                                                null,
                                                val.payload
                                            )
                                        );
                                    }
                                    return [];
                                });
                                var statusText = that.props.sessionStatusText[session.sessionId];
                                var sessionClass = 'session-' + statusText;
                                var sessionStyle = { margin: '1em', padding: '1em', border: '2px solid black', borderRadius: '10px' };
                                var sessionItems = [];
                                var audioItems = [];
                                if (session.audio) {
                                    audioItems = session.audio.map(function (audioData, ikey) {
                                        return React.createElement(
                                            'span',
                                            { key: ikey },
                                            audioData[ikey] && audioData.length > 0 && React.createElement('audio', { src: audioData, controls: true, style: { float: 'right' } })
                                        );
                                    });
                                }
                                if (session.asr) sessionItems = session.asr.map(function (transcript, ikey) {
                                    var slotValues = [];

                                    if (session.intents && session.intents.length > ikey && session.intents[ikey]) slotValues = session.intents[ikey].slots.map(function (slot, skey) {
                                        return React.createElement(
                                            'li',
                                            { key: skey },
                                            slot.slotName.split('_').join(' '),
                                            ' ',
                                            slot.value.value
                                        );
                                    });
                                    return React.createElement(
                                        'div',
                                        { key: ikey },
                                        React.createElement(
                                            'div',
                                            { style: { marginBottom: '1em', fontWeight: 'bold' } },
                                            transcript.text
                                        ),
                                        session.tts && session.tts.length > 0 && React.createElement(
                                            'div',
                                            null,
                                            React.createElement('hr', { style: { height: '1px', width: '100%' } }),
                                            '                               ',
                                            React.createElement(
                                                'div',
                                                null,
                                                React.createElement(
                                                    'i',
                                                    null,
                                                    session.tts && session.tts.length > ikey && session.tts[ikey] && session.tts[ikey].text
                                                )
                                            )
                                        ),
                                        React.createElement(
                                            'div',
                                            null,
                                            React.createElement('hr', { style: { height: '1px', width: '100%' } })
                                        ),
                                        React.createElement(
                                            'div',
                                            null,
                                            slotValues && React.createElement(
                                                'ul',
                                                null,
                                                slotValues
                                            )
                                        )
                                    );
                                });
                                if (session.started && session.sessionId) {
                                    return React.createElement(
                                        'div',
                                        { className: sessionClass, style: sessionStyle, key: sessionLoopKey },
                                        React.createElement(
                                            'button',
                                            { style: { display: 'inline', paddingRight: '0.5em' }, onClick: function onClick(e) {
                                                    return that.toggleLogExpansion(e, session.sessionId);
                                                } },
                                            '+'
                                        ),
                                        React.createElement(
                                            'h4',
                                            { style: { marginLeft: '1em', display: 'inline', clear: 'right' } },
                                            session.sessionId,
                                            ' ',
                                            that.props.sessionStatusText[session.sessionId],
                                            ' '
                                        ),
                                        React.createElement(
                                            'div',
                                            { style: { float: 'right' } },
                                            audioItems
                                        ),
                                        React.createElement('hr', { style: { height: '1px', width: '100%' } }),
                                        React.createElement(
                                            'div',
                                            null,
                                            sessionItems
                                        ),
                                        React.createElement('hr', { style: { height: '1px', width: '100%' } }),
                                        that.isLogExpanded(session.sessionId) && React.createElement(
                                            'div',
                                            null,
                                            logs
                                        )
                                    );
                                }
                                //
                            }
                            return [];
                        });
                        var activityStyle = { padding: '0.2em', borderRadius: '5px', float: 'right', marginRight: '4em' };
                        return React.createElement(
                            'div',
                            { style: { margin: '1em', padding: '1em', border: '2px solid black', borderRadius: '10px' }, key: siteKey },
                            siteKey,
                            that.props.hotwordListening[siteKey] && React.createElement(
                                'b',
                                { style: Object.assign({ backgroundColor: 'lightpink', border: '1px solid red' }, activityStyle) },
                                'Hotword'
                            ),
                            that.props.audioListening[siteKey] && React.createElement(
                                'b',
                                { style: Object.assign({ backgroundColor: 'lightgreen', border: '1px solid green' }, activityStyle) },
                                'Listening'
                            ),
                            React.createElement(
                                'div',
                                null,
                                sessionsRendered
                            )
                        );
                    }
                    return;
                });
                return React.createElement(
                    'b',
                    null,
                    sitesRendered,
                    React.createElement('br', null)
                );
            } else {
                return [];
            }
        }
    }]);
    return SnipsReactLogger;
}(Component);

/* global window */

var SnipsReactFlatLogger = function (_Component) {
    inherits(SnipsReactFlatLogger, _Component);

    function SnipsReactFlatLogger(props) {
        classCallCheck(this, SnipsReactFlatLogger);

        var _this = possibleConstructorReturn(this, (SnipsReactFlatLogger.__proto__ || Object.getPrototypeOf(SnipsReactFlatLogger)).call(this, props));

        _this.state = { showLogMessages: {} };
        _this.toggleMessageExpansion = _this.toggleMessageExpansion.bind(_this);
        _this.isLogMessageExpanded = _this.isLogMessageExpanded.bind(_this);

        return _this;
    }

    createClass(SnipsReactFlatLogger, [{
        key: 'toggleMessageExpansion',
        value: function toggleMessageExpansion(e, key) {
            var showLogMessages = this.state.showLogMessages;
            if (this.isLogMessageExpanded(key)) {
                showLogMessages[key] = false;
            } else {
                showLogMessages[key] = true;
            }
            this.setState({ showLogMessages: showLogMessages });
        }
    }, {
        key: 'isLogMessageExpanded',
        value: function isLogMessageExpanded(key) {
            if (this.state.showLogMessages.hasOwnProperty(key) && this.state.showLogMessages[key]) {
                return true;
            }
            return false;
        }
    }, {
        key: 'render',
        value: function render() {
            var that = this;
            if (this.props.messages) {
                var logs = that.props.messages.map(function (val, key) {
                    return React.createElement(
                        'div',
                        { key: key },
                        React.createElement(
                            'button',
                            { onClick: function onClick(e) {
                                    return that.toggleMessageExpansion(e, key);
                                } },
                            '+'
                        ),
                        '\xA0\xA0',
                        val.text,
                        that.isLogMessageExpanded(key) && React.createElement(
                            'pre',
                            null,
                            val.payload
                        )
                    );
                });
                return React.createElement(
                    'div',
                    null,
                    logs
                );
            }
        }
    }]);
    return SnipsReactFlatLogger;
}(Component);

/* global navigator */
// Polyfill: mediaDevices.
// Not work on Desktop Safari, IE.
// Not work on Mobile browsers.

var nav = {};
nav.mediaDevices = function () {
    if (navigator.mediaDevices) {
        return navigator.mediaDevices;
    }

    navigator.getUserMedia = navigator.getUserMedia || navigator.webkitGetUserMedia || navigator.mozGetUserMedia || navigator.msGetUserMedia;
    if (navigator.getUserMedia) {
        return {
            getUserMedia: function getUserMedia(c) {
                return new Promise(function (y, n) {
                    navigator.getUserMedia.call(navigator, c, y, n);
                });
            }
        };
    }
}();
if (!nav.mediaDevices) {
    alert("mediaDevices() not supported.");
    throw new Error("mediaDevices() not supported.");
}

// Polyfill: AudioContext.
window.AudioContext = window.AudioContext || window.webkitAudioContext || window.mozAudioContext || window.msAudioContext;

var AudioMeter = function (_Component) {
    inherits(AudioMeter, _Component);

    function AudioMeter(props) {
        classCallCheck(this, AudioMeter);

        var _this = possibleConstructorReturn(this, (AudioMeter.__proto__ || Object.getPrototypeOf(AudioMeter)).call(this, props));

        _this.state = {
            volume: 0,
            debug: false
        };
        return _this;
    }

    createClass(AudioMeter, [{
        key: "componentDidMount",
        value: function componentDidMount() {

            // Processing.
            var process = function (event) {
                var buf = event.inputBuffer.getChannelData(0);
                var sum = 0;
                var x;

                for (var i = 0; i < buf.length; i++) {
                    x = buf[i];
                    sum += x * x;
                }

                var rms = Math.sqrt(sum / buf.length);
                this.setState({
                    volume: Math.max(rms, this.state.volume * this.averaging)
                });
                //console.log('Volume: ' + this.state.volume);
                if (this.state.volume > (this.props.volumeWarning > 0 ? this.props.volumeWarning : 0.25)) this.canvasCtx.fillStyle = this.props.style.tooLoudColor ? this.props.style.tooLoudColor : '#FF0000';else this.canvasCtx.fillStyle = this.props.style.color ? this.props.style.color : '#00FF48';
                this.canvasCtx.clearRect(0, 0, this.canvasCtx.canvas.width, this.canvasCtx.canvas.height);
                this.canvasCtx.fillRect(0, this.canvasCtx.canvas.height * (1 - this.state.volume * 4), this.canvasCtx.canvas.width, this.canvasCtx.canvas.height);
            }.bind(this);

            // Init processing.
            nav.mediaDevices.getUserMedia({
                audio: true
            }).then(function (stream) {
                var audioCtx = new AudioContext();
                var source = audioCtx.createMediaStreamSource(stream);
                var processor = audioCtx.createScriptProcessor(256);
                var gainNode = audioCtx.createGain();
                gainNode.gain.value = this.props.inputvolume > 0 ? this.props.inputvolume / 100 : 0.5;
                //console.log(['IV',this.props.inputvolume]);
                this.averaging = 0.95;
                this.canvasCtx = this.refs.canvas.getContext('2d');
                this.canvasCtx.fillStyle = this.props.style.color ? this.props.style.color : '#00FF48';

                processor.onaudioprocess = process;
                processor.connect(audioCtx.destination);
                gainNode.connect(processor);
                source.connect(gainNode);
                if (this.props.addInputGainNode) this.props.addInputGainNode(gainNode);
            }.bind(this)).catch(function (err) {
                console.log('Error occurred while initalizing audio input: ' + err.toString());
            });
        }
    }, {
        key: "render",
        value: function render() {

            var canvasStyle = this.props.style ? Object.assign({}, this.props.style) : {};
            canvasStyle.height = this.props.style.height > 0 ? this.props.style.height : 78;
            canvasStyle.width = this.props.style.width > 0 ? this.props.style.width : 30;
            return React.createElement("canvas", { style: canvasStyle, ref: "canvas", width: canvasStyle.width, height: canvasStyle.height });
        }
    }]);
    return AudioMeter;
}(Component);

var SnipsReactConfig = function (_SnipsReactComponent) {
    inherits(SnipsReactConfig, _SnipsReactComponent);

    function SnipsReactConfig(props) {
        classCallCheck(this, SnipsReactConfig);

        var _this = possibleConstructorReturn(this, (SnipsReactConfig.__proto__ || Object.getPrototypeOf(SnipsReactConfig)).call(this, props));

        if (!props.siteId || props.siteId.length === 0) {
            throw "Config must be configured with a siteId property";
        }
        var eventFunctions = {};
        _this.hideConfig = _this.hideConfig.bind(_this);
        _this.logger = _this.connectToLogger(props.logger, eventFunctions);
        _this.resetConfig = _this.resetConfig.bind(_this);
        return _this;
    }

    /**
    * Activate on mount if user has previously enabled.
    */


    createClass(SnipsReactConfig, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            //this.initSpeechSynthesis.bind(this)();
            this.loadSpeechSynthesisVoices.bind(this)();

            // if previously activated, restore microphone
            //if (localStorage.getItem(this.appendUserId('snipsmicrophone_enabled',this.props.user)) === 'true') {
            //this.activate(false);
            //}
        }

        /**
         * Send Mqtt message to end the session immediately
         */

    }, {
        key: 'sendTestSay',
        value: function sendTestSay(e) {
            e.preventDefault();
            this.sendMqtt("hermes/tts/say", { siteId: this.props.siteId, text: 'This is a test to hear how I speak.' });
        }
    }, {
        key: 'resetConfig',
        value: function resetConfig(e) {
            e.preventDefault();
            var newConfig = this.getDefaultConfig();
            this.setState({ 'config': newConfig });
            localStorage.setItem(this.appendUserId('snipsmicrophone_config', this.props.user), JSON.stringify(newConfig));
            if (this.props.configurationChange) this.props.configurationChange(newConfig);
        }
    }, {
        key: 'getDefaultConfig',
        value: function getDefaultConfig() {
            //console.log(['GDC',this.state]);
            return {
                inputvolume: '70',
                outputvolume: '70',
                voicevolume: '70',
                ttsvoice: 'default', //this.state.voices && this.state.voices.length > 0 ? this.state.voices[0].name :
                voicerate: '50',
                voicepitch: '50',
                hotword: 'browser:oklamp',
                hotwordsensitivity: '50',
                enabletts: 'yes',
                enableaudio: 'yes',
                enablenotifications: 'yes'
            };
        }
    }, {
        key: 'appendUserId',
        value: function appendUserId(text, user) {
            if (user && user._id) {
                return text + "_" + user._id;
            } else {
                return text;
            }
        }
    }, {
        key: 'configurationChange',
        value: function configurationChange(e) {
            var that = this;
            //console.log(['configurationChange',this,e,e.target.value,e.target.id]);
            var config = this.props.config;
            config[e.target.id] = e.target.value;
            this.setState({ config: config });
            // set silence threshhold directly
            if (e.target.id === "inputvolume") {
                // update all input gain nodes
                this.props.inputGainNodes.map(function (node) {
                    //console.log(['set gain',node,that.props.config.inputvolume/100]);
                    node.gain.value = that.props.config.inputvolume / 100;
                });
            }
            localStorage.setItem(this.appendUserId('snipsmicrophone_config', this.props.user), JSON.stringify(config));
            if (this.props.configurationChange) this.props.configurationChange(this.props.config);
        }
    }, {
        key: 'loadSpeechSynthesisVoices',
        value: function loadSpeechSynthesisVoices() {
            var that = this;
            if ('speechSynthesis' in window) {
                // Fetch the list of voices and populate the voice options.
                var loadVoices = function loadVoices() {
                    // Fetch the available voices.
                    var voices = speechSynthesis.getVoices();

                    // Loop through each of the voices
                    var voiceOptions = [];
                    voices.forEach(function (voice, i) {
                        // Create a new option element.
                        voiceOptions.push({ 'name': voice.name, label: voice.name });
                    });
                    voiceOptions.push({ 'name': 'default', label: 'Browser Generated' });
                    that.setState({ voices: voiceOptions });
                    //console.log(['VOICES a',voiceOptions]);
                };

                // Execute loadVoices.


                loadVoices();

                // Chrome loads voices asynchronously.
                window.speechSynthesis.onvoiceschanged = function (e) {
                    loadVoices();
                };
            } else {
                var voiceOptions = [];
                voiceOptions.push({ 'name': 'default', label: 'Browser Generated' });
                that.setState({ voices: voiceOptions });
            }
            // console.log(['LOADED VOICES',this.state.voices]);
        }
    }, {
        key: 'hideConfig',
        value: function hideConfig() {
            if (this.props.hideConfig) this.props.hideConfig();
        }
    }, {
        key: 'render',
        value: function render() {
            var voiceOptions = this.state.voices && this.state.voices.map(function (voice) {
                return React.createElement(
                    'option',
                    { key: voice.name, value: voice.name },
                    voice.label
                );
            });

            var resetIcon = React.createElement(
                'svg',
                { 'aria-hidden': 'true', style: { height: '1.1em' }, role: 'img', xmlns: 'http://www.w3.org/2000/svg', viewBox: '0 0 512 512' },
                React.createElement('path', { fill: 'currentColor', d: 'M500.333 0h-47.411c-6.853 0-12.314 5.729-11.986 12.574l3.966 82.759C399.416 41.899 331.672 8 256.001 8 119.34 8 7.899 119.526 8 256.187 8.101 393.068 119.096 504 256 504c63.926 0 122.202-24.187 166.178-63.908 5.113-4.618 5.354-12.561.482-17.433l-33.971-33.971c-4.466-4.466-11.64-4.717-16.38-.543C341.308 415.448 300.606 432 256 432c-97.267 0-176-78.716-176-176 0-97.267 78.716-176 176-176 60.892 0 114.506 30.858 146.099 77.8l-101.525-4.865c-6.845-.328-12.574 5.133-12.574 11.986v47.411c0 6.627 5.373 12 12 12h200.333c6.627 0 12-5.373 12-12V12c0-6.627-5.373-12-12-12z' })
            );

            var inputStyle = { marginBottom: '0.5em', fontSize: '0.9em' };
            var config = this.props.config;
            return React.createElement(
                'div',
                null,
                React.createElement(
                    'div',
                    { style: { minHeight: '25em', margin: '1em', padding: '1em', width: '90%', border: '2px solid black', borderRadius: '10px', backgroundColor: 'white', zIndex: '9' } },
                    React.createElement(
                        'button',
                        { style: { float: 'right', fontSize: '1.6em', fontWeight: 'bold', border: '2px solid black', borderRadius: '50px' }, onClick: this.hideConfig },
                        'X'
                    ),
                    React.createElement(
                        'div',
                        { style: { float: 'left', marginRight: '2em' } },
                        status >= 2 && React.createElement(
                            'span',
                            { onClick: this.deactivate },
                            React.createElement(
                                'button',
                                { className: 'btn btn-danger', style: { fontSize: '1.5em' } },
                                ' ',
                                stopIcon2,
                                ' Disable '
                            )
                        )
                    ),
                    React.createElement(
                        'h1',
                        null,
                        'Microphone Configuration'
                    ),
                    React.createElement(
                        'form',
                        { style: { fontSize: '1.8em' } },
                        React.createElement(
                            'div',
                            { style: { clear: 'both', width: '100%' } },
                            React.createElement(
                                'div',
                                { style: { float: 'right' } },
                                React.createElement(AudioMeter, { inputvolume: this.props.config.inputvolume, addInputGainNode: this.props.addInputGainNode, source: this.source, style: { float: 'right', marginRight: "2em", height: '200', width: '50', dtooLoudColor: "#FF9800", scolor: '#889bd8', border: '1px solid black', backgroundColor: 'lightgrey' } })
                            ),
                            React.createElement(
                                'div',
                                { style: { width: '80%' } },
                                React.createElement(
                                    'div',
                                    { className: 'form-group' },
                                    React.createElement(
                                        'b',
                                        { style: { marginBottom: '0.8em' } },
                                        'Volume\xA0\xA0\xA0'
                                    )
                                ),
                                React.createElement(
                                    'div',
                                    { className: 'form-group' },
                                    React.createElement(
                                        'label',
                                        { htmlFor: 'inputvolume' },
                                        'Microphone '
                                    ),
                                    React.createElement('input', { type: 'range', id: 'inputvolume', value: config.inputvolume, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle), min: '0', max: '200' })
                                ),
                                React.createElement(
                                    'div',
                                    { className: 'form-group' },
                                    React.createElement(
                                        'label',
                                        { htmlFor: 'outputvolume' },
                                        'Output '
                                    ),
                                    React.createElement('input', { type: 'range', id: 'outputvolume', value: config.outputvolume, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle) })
                                ),
                                React.createElement(
                                    'div',
                                    { className: 'form-group' },
                                    React.createElement(
                                        'label',
                                        { htmlFor: 'voicevolume' },
                                        'Voice '
                                    ),
                                    React.createElement('input', { type: 'range', id: 'voicevolume', value: config.voicevolume, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle) })
                                )
                            )
                        ),
                        React.createElement(
                            'div',
                            { style: { clear: 'both', width: '100%' }, className: 'form-group' },
                            React.createElement('hr', { style: { width: '100%' } })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement(
                                'label',
                                { htmlFor: 'hotword' },
                                'Hotword '
                            ),
                            React.createElement(
                                'select',
                                { style: inputStyle, id: 'hotword', value: config.hotword, onChange: this.configurationChange.bind(this) },
                                React.createElement(
                                    'option',
                                    { value: 'browser:ok lamp' },
                                    'OK Lamp (Browser)'
                                ),
                                React.createElement(
                                    'option',
                                    { value: 'browser:navy blue' },
                                    'Navy Blue (Browser)'
                                ),
                                React.createElement(
                                    'option',
                                    { value: 'disabled' },
                                    'Disabled'
                                )
                            )
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement(
                                'label',
                                { htmlFor: 'hotwordsensitivity' },
                                'Hotword Sensitivity '
                            ),
                            React.createElement('input', { type: 'range', id: 'hotwordsensitivity', value: config.hotwordsensitivity, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle) })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement('hr', { style: { width: '100%' } })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement(
                                'b',
                                { style: { marginBottom: '0.8em' } },
                                'Notifications\xA0\xA0\xA0'
                            )
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-inline' },
                            React.createElement(
                                'label',
                                { htmlFor: 'enabletts' },
                                ' Voice '
                            ),
                            React.createElement(
                                'select',
                                { style: inputStyle, id: 'enabletts', value: config.enabletts, onChange: this.configurationChange.bind(this) },
                                React.createElement(
                                    'option',
                                    { value: 'yes' },
                                    'Yes'
                                ),
                                React.createElement(
                                    'option',
                                    { value: 'no' },
                                    'No'
                                )
                            ),
                            React.createElement(
                                'label',
                                { htmlFor: 'enableaudio' },
                                ' Audio '
                            ),
                            React.createElement(
                                'select',
                                { style: inputStyle, id: 'enableaudio', value: config.enableaudio, onChange: this.configurationChange.bind(this) },
                                React.createElement(
                                    'option',
                                    { value: 'yes' },
                                    'Yes'
                                ),
                                React.createElement(
                                    'option',
                                    { value: 'no' },
                                    'No'
                                )
                            ),
                            React.createElement(
                                'label',
                                { htmlFor: 'enablenotifications' },
                                ' Screen '
                            ),
                            React.createElement(
                                'select',
                                { style: inputStyle, id: 'enablenotifications', value: config.enablenotifications, onChange: this.configurationChange.bind(this) },
                                React.createElement(
                                    'option',
                                    { value: 'yes' },
                                    'Yes'
                                ),
                                React.createElement(
                                    'option',
                                    { value: 'no' },
                                    'No'
                                )
                            )
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement('hr', { style: { width: '100%' } })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-inline' },
                            React.createElement(
                                'label',
                                { htmlFor: 'ttsvoice' },
                                'Voice '
                            ),
                            React.createElement(
                                'select',
                                { style: inputStyle, id: 'ttsvoice', value: config.ttsvoice, onChange: this.configurationChange.bind(this) },
                                voiceOptions
                            ),
                            '\xA0\xA0\xA0',
                            React.createElement(
                                'button',
                                { className: 'btn btn-success', style: { fontSize: '1em' }, onClick: this.sendTestSay.bind(this) },
                                'Test'
                            )
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement(
                                'label',
                                { htmlFor: 'voicerate' },
                                'Rate '
                            ),
                            React.createElement('input', { type: 'range', id: 'voicerate', value: config.voicerate, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle) })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement(
                                'label',
                                { htmlFor: 'voicepitch' },
                                'Pitch '
                            ),
                            React.createElement('input', { type: 'range', id: 'voicepitch', value: config.voicepitch, onChange: this.configurationChange.bind(this), style: Object.assign({ width: '80%' }, inputStyle) })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement('hr', { style: { width: '100%' } }),
                            React.createElement(
                                'span',
                                { onClick: this.resetConfig },
                                React.createElement(
                                    'button',
                                    { className: 'btn btn-danger', style: { fontSize: '1em' } },
                                    ' ',
                                    resetIcon,
                                    ' Reset Configuration'
                                )
                            ),
                            React.createElement('hr', { style: { width: '100%' } })
                        ),
                        React.createElement(
                            'div',
                            { className: 'form-group' },
                            React.createElement('br', null),
                            React.createElement('br', null),
                            React.createElement('br', null)
                        )
                    )
                )
            );
        }
    }]);
    return SnipsReactConfig;
}(SnipsReactComponent);

var SnipsReactSatellite = function (_Component) {
    inherits(SnipsReactSatellite, _Component);

    function SnipsReactSatellite(props) {
        classCallCheck(this, SnipsReactSatellite);

        var _this = possibleConstructorReturn(this, (SnipsReactSatellite.__proto__ || Object.getPrototypeOf(SnipsReactSatellite)).call(this, props));

        _this.siteId = props.siteId ? props.siteId : 'browser_' + parseInt(Math.random() * 100000000, 10);
        _this.inputGainNodes = [];
        _this.state = { showConfig: false, config: {} };
        _this.setLogData = _this.setLogData.bind(_this);
        _this.setConfig = _this.setConfig.bind(_this);
        _this.showConfig = _this.showConfig.bind(_this);
        _this.hideConfig = _this.hideConfig.bind(_this);
        _this.addInputGainNode = _this.addInputGainNode.bind(_this);

        _this.logger = props.logger ? props.logger : new SnipsLogger(Object.assign({ logAudio: false, setLogData: _this.setLogData }, props));
        var configString = localStorage.getItem(_this.appendUserId('snipsmicrophone_config', props.user));
        var config = null;
        try {
            config = JSON.parse(configString);
        } catch (e) {}
        if (config) {
            _this.state.config = config;
        } else {
            // default config
            var newConfig = _this.getDefaultConfig();
            _this.state.config = newConfig;
            localStorage.setItem(_this.appendUserId('snipsmicrophone_config', _this.props.user), JSON.stringify(newConfig));
        }
        return _this;
    }

    createClass(SnipsReactSatellite, [{
        key: 'appendUserId',
        value: function appendUserId(text, user) {
            if (user && user._id) {
                return text + "_" + user._id;
            } else {
                return text;
            }
        }
    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {}
    }, {
        key: 'setLogData',


        // force update
        value: function setLogData(sites, messages, sessionStatus, sessionStatusText, hotwordListening, audioListening) {
            this.setState(this.state);
        }
    }, {
        key: 'setConfig',
        value: function setConfig(config) {
            this.setState({ config: config });
        }
    }, {
        key: 'showConfig',
        value: function showConfig() {
            this.setState({ showConfig: true });
        }
    }, {
        key: 'hideConfig',
        value: function hideConfig() {
            this.setState({ showConfig: false });
        }
    }, {
        key: 'addInputGainNode',
        value: function addInputGainNode(node) {
            this.inputGainNodes.push(node);
        }
    }, {
        key: 'getDefaultConfig',
        value: function getDefaultConfig() {
            //console.log(['GDC',this.state]);
            return {
                inputvolume: '70',
                outputvolume: '70',
                voicevolume: '70',
                ttsvoice: 'default', //this.state.voices && this.state.voices.length > 0 ? this.state.voices[0].name :
                voicerate: '50',
                voicepitch: '50',
                hotword: 'browser:oklamp',
                hotwordsensitivity: '50',
                enabletts: 'yes',
                enableaudio: 'yes',
                enablenotifications: 'yes'
            };
        }
    }, {
        key: 'render',


        //  
        value: function render() {
            var position = this.props.position ? this.props.position : 'top left';
            return React.createElement(
                'div',
                { id: 'snipsreactsatellite' },
                React.createElement(SnipsReactHotwordServer, _extends({}, this.props, { logger: this.logger, siteId: this.siteId, config: this.state.config, addInputGainNode: this.addInputGainNode })),
                React.createElement(SnipsReactMicrophone, _extends({}, this.props, { position: position, logger: this.logger, siteId: this.siteId, config: this.state.config, showConfig: this.showConfig, hideConfig: this.hideConfig, addInputGainNode: this.addInputGainNode })),
                React.createElement(SnipsReactTts, _extends({}, this.props, { logger: this.logger, siteId: this.siteId, config: this.state.config })),
                React.createElement(SnipsReactSpeaker, _extends({}, this.props, { logger: this.logger, siteId: this.siteId, config: this.state.config })),
                this.props.intents && React.createElement(SnipsAppServer, _extends({}, this.props, { logger: this.logger, siteId: this.siteId, config: this.state.config })),
                React.createElement(
                    'div',
                    { style: { width: '100%', clear: 'both' } },
                    '\xA0'
                ),
                this.state.showConfig && React.createElement(SnipsReactConfig, _extends({}, this.props, { setConfig: this.setConfig, configurationChange: this.setConfig, hideConfig: this.hideConfig, config: this.state.config, addInputGainNode: this.addInputGainNode, inputGainNodes: this.inputGainNodes }))
            );
        }
    }]);
    return SnipsReactSatellite;
}(Component);

export { SnipsMqttServer, SnipsLogger$1 as SnipsLogger, SnipsReactComponent, SnipsReactMicrophone, SnipsReactSpeaker, SnipsAppServer as SnipsReactAppServer, SnipsReactHotwordServer, SnipsReactTts, SnipsReactLogger, SnipsReactFlatLogger, SnipsReactSatellite };
//# sourceMappingURL=index.es.js.map
